console.log( 'crossword.js' );

var crosswordPage = (function() {

  var DOWN = 0;
  var ACROSS = 1;
  var STARTS_HERE = 2;
  var CANT_GO_DOWN = '|';
  var CANT_GO_ACROSS = '_';
  var CANT_GO_DOWN_OR_ACROSS = '+';
  var BLANK = '@';
  var txtDown = "Down" ;
  var txtAcross = "Across";
  var txtAnswer = "Answer:";
  var txtShowAnswer = "Show Answer";
  var txtCheck = "Check" ;
  var MAX_WORD_LENGTH = 30;
  
  var width ;
  var height ;
  var blockSize ; 
  var numRows;
  var numColumns ;


function handleKeyPress(evt) {
  console.log( 'crossword handleKeyPress' );
  evt = evt || window.event;
  //console.log( 'onkeypress for ' + evt.which );
  var charCode = evt.which || evt.keyCode;
  //console.log( 'onkeypress ' + charCode );
  if ( charCode == ENTER_KEY ) {
      peek();
  }
  else {
      var charStr = String.fromCharCode(charCode);
      if ( charStr.length == 1 ) {
          crosswordPage.pressKey( charStr.toUpperCase() );
      }
  }
};
   
   function isBlank( ch ) 
   {
       return ch == BLANK || ch == CANT_GO_DOWN || ch == CANT_GO_ACROSS || ch == CANT_GO_DOWN_OR_ACROSS ;
   }
   
   
   function getAnsChar( row, col )
   {
       if ( row < 0 || row >= puzzle.numRows || col < 0 || col >= puzzle.numColumns ) {
           return BLANK ;
       }
       return puzzle.answer.get( row, col );
   }
   
   function writePuzzle( puzzle )
   {
      var row;
      var col;
      var html = '' ;
      
      html += "<style>" ;
      html += ".sq { width: " + puzzle.blockSize + "px; height: " + puzzle.blockSize + "px; line-height: " + puzzle.blockSize + "px; }" ;
      html += ".cwlet { font-size: " + (puzzle.blockSize-5) + "px; }" ;
      html += "</style>" ;
      
      html += '<table border="1" class="puzzleTable">' ;
      for ( row = 0; row<puzzle.numRows; row++ ) {
          html += "<tr>" ;
          for ( col = 0; col<puzzle.numColumns; col++ ) {
              var ansChar = getAnsChar( row, col );
              if ( isBlank( ansChar ) ) {
                   html += "<td class='blank'>&nbsp;</td>" ;
              }
              else {
                  var ch = puzzle.guess.get( row, col );
                  var pressAction = "crosswordPage.pressSquare(" + row + "," + col +", event)";
                  var releaseAction = "crosswordPage.releaseSquare(" + row + "," + col +", event)";
                  var actionHtml = "onmousedown='" + pressAction + "' onmouseup='" + releaseAction + "' "
                                 + "ontouchstart='" + pressAction + "' ontouchend='" + releaseAction + "' " ; 
	              html += "<td " + actionHtml + " id='sq" + col + "_" + row + "' class='sq'>" ;
		          html +=     "<div class='cwnum' id='num" + col + "_" + row + "'></div>" ;
	              html +=   "<div class='block'>" ;
                  html +=     "<div class='cwlet' id='let" + col + "_" + row + "'>" ;
	              if ( ch != BLANK ) { html +=      ch; }
                  html +=      "</div>" ;
                  html +=   "</div>" ;
	              html += "</td>" ;
              }
          }
          html += "</tr>" ;
      }
      html += "</table>" ;
      
      
      return html ;
   }
   
   function keydown( event ) {
       console.log( 'keydown for ' + event.which );
       if ( event.which == BACKSPACE_KEY ) {
            backspace( event );
       }
       else if ( event.which == UP_ARROW_KEY || event.which == DOWN_ARROW_KEY || event.which == LEFT_ARROW_KEY || event.which == RIGHT_ARROW_KEY ) {
            processArrowKey( event.which ) ;
       }
   }
   
   function processArrowKey( key ) {
       if ( curRow<0 || curCol<0 ) return ;
       var newRow = curRow ;
       var newCol = curCol ;
       if ( key == UP_ARROW_KEY && newRow > 0) {
            newRow -- ;    
       }
       else if ( key == DOWN_ARROW_KEY ) {
            newRow ++ ;
       }
       else if ( key == LEFT_ARROW_KEY ) {
            newCol -- ;
       }
       else if ( key == RIGHT_ARROW_KEY ) {
            newCol ++ ;
       }
       var squareField = getSquareElement( newRow, newCol );
       if ( squareField != null ) {
           clickSquare( newRow, newCol );
       }
   }
   
   var pressStartTime = null ;
   
   function pressSquare( row, col, event ) {
	    pressStartTime = new Date().getTime();
        killEvent( event );
        clickSquare( row, col );
   }
   
   function releaseSquare( row, col, event ) {
	    var minDelay = 1000 - (new Date().getTime() - pressStartTime);
	    if (  minDelay>0 ) {
           setTimeout( hideClue, minDelay );
	    }
	    else {
	    	hideClue();
	    }
        killEvent( event );
   }
   
   function clickSquare( row, col)
   {
      var newWord = null ;
      
      if ( row == curRow && col == curCol && currentHilightedWord != null ) {
           // see if there is a word here in the opposite direction
           var oppositeDirection = (currentHilightedWord.direction == DOWN) ? ACROSS : DOWN;
           newWord = puzzle.findWordAt( row, col, oppositeDirection );
      }
      
      if ( newWord == null ) {
           newWord = puzzle.findWordAt( row, col, STARTS_HERE );
      }
      
      if ( newWord == null ) {
          newWord = puzzle.findWordAt( row, col, ACROSS );
      }
      
      if ( newWord == null ) {
           newWord = puzzle.findWordAt( row, col, DOWN );
       }    
      
      
      setCurrentSquare( -1, -1 ) ;
      uncheckAnswers() ;
      
      hilightNewWord( newWord );
      setCurrentSquare( row, col );
   }
   
   
   var curRow = -1;
   var curCol = -1;
   
   function setCurrentSquare( row, col ) {
       if ( curRow >= 0 ) {
            var squareField = getSquareElement( curRow, curCol );
            if ( currentHilightedWord == puzzle.findWordAt( curRow, curCol, currentHilightedWord.direction ) ) {
                squareField.className = "sq hilightSq" ;
            }
            else {
                squareField.className = "sq" ;
            }
            curRow = -1;
            curCol = -1;
       }
       curRow = row ;
       curCol = col ;
       if ( curRow >= 0 ) {
           var squareField = getSquareElement( curRow, curCol );
           squareField.className = "sq curSq" ;
           if ( !isDone() ) {
           	   showKeyboard(1);
           }
       }
   }
   
   

   function Puzzle_findWordAt( row, col, direction ) 
   {
       var i ;
       var puzzleWord ;
       for ( i=0; i<this.placedWords.length; i++ ) {
           puzzleWord = this.placedWords[i] ;
           
           if ( direction == STARTS_HERE ) {
                if ( puzzleWord.row == row && puzzleWord.col == col ) {
                     this.curWordNumber = i;
                     return puzzleWord ;
                }
           }
           
           if ( puzzleWord.direction == direction ) {
                if ( direction == ACROSS ) {
                    if ( row == puzzleWord.row ) {
                        if ( col >= puzzleWord.col && col<(puzzleWord.col + puzzleWord.word.length) ) {
                             this.curWordNumber = i;
                             return puzzleWord ;
                        }
                    }
                }
                else {
                    if ( col == puzzleWord.col ) {
                         if ( row >= puzzleWord.row && row < (puzzleWord.row + puzzleWord.word.length) ) {
                              this.curWordNumber = i;
                              return puzzleWord ;
                         }
                    }
                }
                
           }
       }
       return null ;
   }

   function doNothing()
   {
   }
   

   function getLetterElement( row, col ) 
   {
       return letterFields[row][col];
       //return document.getElementById( "let" + col + "_" + row );
   }
   
    function getSquareElement( row, col )
    {
        if ( row<0 || row>=squareFields.length || col<0 || col>=squareFields[row].length ) {
            return null ;
        }
        var elem = squareFields[row][col];
        if ( elem == null ) {
            console.log( 'getSquareElement for ' + row + ',' + col + ' returning null ' );
        }
        return elem ; 
    }
   
   
   function showLetter( col, row, ch )
   {
       var letterField = getLetterElement( row, col );
       letterField.innerHTML = ch ;
       puzzle.guess.set( row, col, ch );
   }
   

   function showSolution()
   {
      checkAnswers();
      var row;
      var col;

      for ( row = 0; row<puzzle.numRows; row++ ) {
          for ( col = 0; col<puzzle.numColumns; col++ ) {
              var ansChar = puzzle.answer.get( row, col );
              if (!isBlank( ansChar ) ) {
                  showLetter( col, row, ansChar );
              }
          }
      }
      puzzle.shownSolution = true ;
      showAllDone();
   }
   
   function undoSolution()
   {
      var row;
      var col;

      for ( row = 0; row<puzzle.numRows; row++ ) {
          for ( col = 0; col<puzzle.numColumns; col++ ) {
    	      var letterField = getLetterElement( row, col );
    	      if ( letterField != null ) {
   	          	 letterField.innerHTML = letterField.guessedChar ? letterField.guessedChar : '';
                 letterField.className = 'cwlet' ; 
   	          }
          }
      }
      puzzle.shownSolution = false ;
      showHelp();
   }
   

   var numWrong = 0;
   var numCorrect = 0;
   
   function checkAnswers() 
   {
      clearCurrentWord();
      var row;
      var col;

      numWrong = 0;
      numCorrect = 0;
      for ( row = 0; row<puzzle.numRows; row++ ) {
          for ( col = 0; col<puzzle.numColumns; col++ ) {
              var ansChar = puzzle.answer.get( row, col );
              if (!isBlank( ansChar )) {
			       var letterField = getLetterElement( row, col );
			       var guessedChar = letterField.innerHTML ;
			       letterField.guessedChar = guessedChar ;
			       if ( guessedChar != ansChar ) {
			      	   letterField.className = 'cwlet wrongLet' ; 
			      	   numWrong++;
			       }
			       else {
			           numCorrect++ ;
			       }
              }
          }
      }
   }


   function canUndo() 
   {
      var row;
      var col;

      for ( row = 0; row<puzzle.numRows; row++ ) {
          for ( col = 0; col<puzzle.numColumns; col++ ) {
              var ansChar = puzzle.answer.get( row, col );
              if (!isBlank( ansChar )  ) {
			       var letterField = getLetterElement( row, col );
			       if ( letterField.guessedChar != ansChar ) {
			    	   console.log( 'canUndo guessedChar=' + letterField.guessedChar + ' ansChar=' + ansChar );
			       	   return true ;
			       }
              }
          }
      }
      return false ;
   }


   function isDone() 
   {
      var row;
      var col;

      for ( row = 0; row<puzzle.numRows; row++ ) {
          for ( col = 0; col<puzzle.numColumns; col++ ) {
              var ansChar = puzzle.answer.get( row, col );
              if (!isBlank( ansChar ) ) {
			       var letterField = getLetterElement( row, col );
			       var guessedChar = letterField.innerHTML ;
			       if ( guessedChar != ansChar ) {
			      	   return false ;
			       }
              }
          }
      }
      return true ;
   }
   
   function uncheckAnswers()
   {
      if ( numWrong != 0 ) {
	      var row;
	      var col;
	
	      for ( row = 0; row<puzzle.numRows; row++ ) {
	          for ( col = 0; col<puzzle.numColumns; col++ ) {
	              var ansChar = puzzle.answer.get( row, col );
	              if (!isBlank( ansChar ) ) {
				      var letterField = getLetterElement( row, col );
	  	      	      letterField.className = 'cwlet' ; 
	              }
	          }
	      }
	      numWrong = 0;
	  }
   }
   

   function Array2d( numRows, numColumns )
   {
      var row;
      var col;
      this.data = new Array();

      for ( row = 0; row<numRows; row++ ) {
         var dataArray = new Array();
         for ( col = 0; col<numColumns; col++ ) {
             dataArray[ col ] = BLANK;
         }
         this.data[ row ] = dataArray ;
      }       
      initArray2d( this );
   }
   
   function initArray2d( array2d ) {
      array2d.get = Array2d_get ;
      array2d.set = Array2d_set ;
      array2d.addWord = Array2d_addWord ;
   }


   function Array2d_get( row, col )
   {
       return this.data[row][col] ;
   }

   function Array2d_set( row, col, val )
   {
      this.data[row][col] = val ;
   }

   function Array2d_addWord( word, x, y, direction )
   {
      var i;
      for ( i=0; i<word.length; i++ ) {
         var ch = word.charAt( i ) ;
         this.set( y, x, ch );
         if ( direction == DOWN ) {
            y++;
         }
         else {
            x++ ;
         }
      }
   }






   function PuzzleWord( word, clue, row, col, direction )
   {
      this.word = word ;
      this.clue = clue ;
      this.row = row ;
      this.col = col ;
      this.direction = direction ;
    
      initPuzzleWord( this );  
   }
   
   function initPuzzleWord( puzzleWord ) {
      puzzleWord.compareTo = PuzzleWord_compareTo ;
   }
   
   
   /**
      Compare this PuzzleWord with otherWord to determine
      how they should be sorted.
   **/
   function PuzzleWord_compareTo( otherWord )
   {
	  if ( otherWord.direction != this.direction ) {
	      //return this.direction - otherWord.direction ;
	      return otherWord.direction - this.direction ;
	  }
	  
      if ( this.row != otherWord.row ) {
          return this.row - otherWord.row ;
      }
      else {
          // on the same row, compare columns
          return this.col - otherWord.col ;
      }
   }


   var currentHilightedWord = null ;
   
   function hilightNewWord( puzzleWord )
   {
      if ( currentHilightedWord != null ) {
          hilightWord( currentHilightedWord, false );
      }
      if ( puzzleWord != null ) {
          hilightWord( puzzleWord, true );
      }
      currentHilightedWord = puzzleWord ;
      updateInputArea( puzzleWord );
   }
   
   
   var inputLength ;
   var letterFields ;
   var squareFields ;
   
  function initElementVars()
  {
      var row;
      var col;
      
      letterFields = new Array();
      squareFields = new Array();
      console.log( 'in initElementVars initializing squareFields numRows=' + puzzle.numRows + ' numColumns=' + puzzle.numColumns );
      for ( row = 0; row<puzzle.numRows; row++ ) {
          letterFields[row] = new Array();
          squareFields[row] = new Array();
          for ( col = 0; col<puzzle.numColumns; col++ ) {
              letterFields[row][col] = document.getElementById( "let" + col + "_" + row );
              squareFields[row][col] = document.getElementById( "sq"  + col + "_" + row );
          }
      }
  }
  
  
  function getItemName() {
     return "crosswordPuzzle" + studyStack.id ;
  }
  
  function saveCurrentPuzzle() {
       var str = JSON.stringify( puzzle );
       var itemName = getItemName();
       dataManager.setLocalItem( itemName,  str );
       console.log( 'saved current puzzle ' + itemName + ' as ' + str );
  }
  
  
  function loadPuzzle() {
      puzzle = null ;
      try {
        var itemName = getItemName();
      	var value = dataManager.getLocalItem( itemName );
      	console.log( 'read in ' + itemName + ' as ' + value );
      	puzzle = JSON.parse( value );
      }
      catch ( err ) {
          console.log( 'error ' + err );
      }
      if ( puzzle == null ) {
       	  puzzle = generateNewPuzzle();
       	  if ( puzzle.placedWords.length == 0 ) {
         	   toast( 'There are not enough words in your cards for a crossword puzzle', 4000 );
        	   keyboardArea.style.display = 'none';
       	  }
      }
      else {
          initPuzzle( puzzle );
      }
	   document.getElementById( 'crosswordPage' ).innerHTML = writePuzzleAndClues( puzzle );
	      
       initElementVars();
	   clearCurrentWord();
  }
  
  function clearCurrentWord() {
       curRow = -1;
       curCol = -1;
       hilightNewWord( null );
       currentHilightedWord = null ;
       showHelp();
  }
  
  function generateNewPuzzle( ) {
 	   puzzle = new Puzzle( numRows, numColumns, blockSize );
 	   console.log( 'generateNewPuzzle puzzle=' + puzzle + ' blockSize=' + blockSize + ' numRows=' + numRows + ' numColumns=' + numColumns ); 
       studyStack.loadWords( 8, 40, puzzle );
	   puzzle.placeWords();
	   console.log( 'before trim puzzle = ' + puzzle );
	   puzzle = trimPuzzle( puzzle );
	   return puzzle ;
  }
  
   
   function updateInputArea( puzzleWord )
   {
   	  if ( puzzleWord != null ) {
   	      inputLength = puzzleWord.word.length ;
   	      showClue( escapeHtml( puzzleWord.clue ) );
   	  }
   }
   
   
   
   function hilightWord( puzzleWord, hilight )
   {
   
        var row = puzzleWord.row ;
        var col = puzzleWord.col ;
        var i ;
        var length = puzzleWord.word.length
        
        for ( i=0; i<length; i++ ) {
        
            var squareField = getSquareElement( row, col );
            if ( squareField == null ) { return; }
            squareField.className = (hilight ? "sq hilightSq" : "sq") ;
        
            if ( puzzleWord.direction == DOWN ) {
                row++;
            }
            else {
                col++;
            }
        }
        
   }
   
   
    
   function Puzzle( numRows, numColumns, blockSize )
   {
      this.numRows = numRows ;
      this.numColumns = numColumns ;
      this.blockSize = blockSize ;
      this.answer = new Array2d( numRows, numColumns );
      this.guess = new Array2d( numRows, numColumns );
      this.placedWords = new Array();
      this.unplacedWords = new Array();
      this.numUnplacedWords = 0;
      this.curWordNumber = -1;
      this.shownSolution = false ;
      initPuzzle( this );
  }
  
  function initPuzzle( puzzle ) {
  
      puzzle.placeWord = Puzzle_placeWord ;
      puzzle.placeWords = Puzzle_placeWords ;
      puzzle.placeWordAnywhere = Puzzle_placeWordAnywhere ;
      puzzle.wordFits = Puzzle_wordFits ;
      puzzle.markSpace = Puzzle_markSpace ;
      puzzle.isBlank = Puzzle_isBlank ;
      puzzle.findWordAt = Puzzle_findWordAt ;
      puzzle.addWord = Puzzle_addWord;
      puzzle.removeWord = Puzzle_removeWord ;
      
      for ( var i=0; i<puzzle.placedWords.length; i++ ) {
          initPuzzleWord( puzzle.placedWords[i] );
      }
      
      initArray2d( puzzle.answer );
      initArray2d( puzzle.guess );
  }
/*  
  function createPuzzleFromJSON( json ) {
      initPuzzleMethods( json );
      return json ;
  }
*/

  /**
     Adds a word and its clue to the puzzle at the 
     specified location and direction.
  **/
  function Puzzle_placeWord( word, clue, x, y, direction )
  {
	    if ( word.length == 0 ) {
	    	return ;
	    }
        this.placedWords[ this.placedWords.length ] = new PuzzleWord( word, clue, y, x, direction );
        defaultStatus = this.placedWords.length + " words loaded." ;
        
        this.answer.addWord( word, x, y, direction );
        
        if ( direction == DOWN ) {
             this.markSpace( y-1, x, CANT_GO_DOWN_OR_ACROSS );
             this.markSpace( y + word.length, x, CANT_GO_DOWN_OR_ACROSS );
        }      
        else {
             this.markSpace( y, x-1, CANT_GO_DOWN_OR_ACROSS );
             this.markSpace( y, x+word.length, CANT_GO_DOWN_OR_ACROSS );
        }
        
        for ( i=0; i<word.length; i++ ) {
            if ( direction == DOWN ) {
                this.markSpace( y, x-1, CANT_GO_DOWN );
                this.markSpace( y, x+1, CANT_GO_DOWN );
                y++;
            }
            else {
                this.markSpace( y-1, x, CANT_GO_ACROSS );
                this.markSpace( y+1, x, CANT_GO_ACROSS );
                x++ ;
            }
        }
  }
  
  
  
  
  
  function Puzzle_markSpace( y, x, markAs )
  {
      if ( x >= 0 && x<this.numColumns && y>=0 && y<this.numRows ) {
          var curVal = this.answer.get( y, x );
          if ( (markAs == CANT_GO_DOWN && curVal == CANT_GO_ACROSS ) ||
               ( markAs == CANT_GO_ACROSS && curVal == CANT_GO_DOWN ) ) {
               this.answer.set( y, x, CANT_GO_DOWN_OR_ACROSS );
          }
          else if ( curVal == BLANK ) {
               this.answer.set( y, x, markAs );
          }
      }
  }
  
  function Puzzle_isBlank( y, x ) 
  {
      if ( x >= 0 && x<this.numColumns && y>=0 && y<this.numRows ) {
          var curVal = this.answer.get( y, x );
          return isBlank( curVal );          
      }
      else {
          return true ;
      }
  }
  


  
  /**
     Search for a place where this word fits on the puzzle.
     If it finds a place the word is added to the puzzle.
     If the word would fit in multiple places, it places the word
     in the position where the most letters are shared with 
     existing words.
  **/
  function Puzzle_placeWordAnywhere( word, clue ) 
  {
     word = word.toUpperCase();
      var wordLen = word.length ;
      var random2 = Math.floor( Math.random()* 2) ;
      var d ;
      var mostMatchingLetters = -1 ;
      var bestX;
      var bestY;
      var bestDirection ;
      var maxStartY ;
      var maxStartX ;
      
      for ( d=0; d<2; d++ ) {
          var direction = (d + random2) % 2;
          if ( direction == DOWN ) {
              maxStartY = this.numRows - wordLen ;
              maxStartX = this.numColumns - 1;
          }
          else {
              maxStartX = this.numColumns - wordLen ;
              maxStartY = this.numRows - 1;
          }
          var randomX = Math.floor( Math.random() * maxStartX );
          var randomY = Math.floor( Math.random() * maxStartY );
          for ( startX = 0 ; startX<=maxStartX; startX++ ) {
              for ( startY = 0; startY<=maxStartY; startY++ ) {
                   var x = (startX + randomX) % (maxStartX + 1);
                   var y = (startY + randomY) % (maxStartY + 1);
                   
                   // try to see if word will fit going direction 
                   // starting at x, y
                   var matchingLetters = this.wordFits( word, x, y, direction );
                   if ( (matchingLetters > mostMatchingLetters) && (matchingLetters < wordLen) ) {
                       mostMatchingLetters = matchingLetters ;
                       bestX = x ;
                       bestY = y ;
                       bestDirection = direction ;
                   }
              }
          }
      }
      
      if ( mostMatchingLetters > 0 ) {
           this.placeWord( word, clue, bestX, bestY, bestDirection );
           return true ;
      }
      
      // didn't place word
      return false ;
  }


  function Puzzle_wordFits( word, startX, startY, direction ) 
  {
      var wordLen = word.length ;
      var i;
      var x=startX ;
      var y=startY ;
      var matchingLetters = 0;
      
      if ( direction == DOWN ) {
          if ( !this.isBlank( startY-1, startX ) ) return 0;
          if ( !this.isBlank( startY + wordLen, startX )) return 0;
      }
      else {
          if ( !this.isBlank( startY, startX - 1 ) ) return 0;
          if ( !this.isBlank( startY, startX + wordLen ) ) return 0;
      }

      for ( i=0; i<wordLen; i++ ) {
          var letter = word.charAt( i );
          //defaultStatus = "wordFits: word=" + word + " startX=" + startX + " startY=" + startY + " direction=" + direction + " i=" + i + " x=" + x + " y=" + y ;
          var puzzleChar = this.answer.get( y, x );
          if ( puzzleChar == letter ) {
              matchingLetters ++ ;
          }
          else if ( ! ( (puzzleChar == BLANK)   ||
                        ((direction == DOWN) && (puzzleChar == CANT_GO_ACROSS)) ||
                        ((direction == ACROSS) && (puzzleChar == CANT_GO_DOWN)) ) ) {
                 return -1 ;
 	      }
	      if ( direction == DOWN ) {
	          y++;
	      }
	      else {
	          x++;
	      }
      }
      return matchingLetters ;
  }



  function WordCluePair( word, clue )
  {
       this.word = word ;
       this.clue = clue ;
       this.compareTo = WordCluePair_compareTo ;
  }
  
  function WordCluePair_compareTo( wordClue )
  {
      return this.word.length - wordClue.word.length ;
  }
  
  /**
     addWord adds the word and clue to a list of word/clue pairs that may be used 
     for this puzzle.
  **/
  function Puzzle_addWord( word, clue ) 
  {
    word = removePunctuation( word );
    if ( word.length>MAX_WORD_LENGTH || (word.length > this.numColumns && word.length > this.numRows) ) {
       return false ;
    }
    word = word.toUpperCase() ;
    var i;
    for ( i=0; i<this.numUnplacedWords; i++ ) {
        if ( this.unplacedWords[ i ].word == word ) {
            // word already in list
            return false ;
        }
    }
	this.unplacedWords[ this.numUnplacedWords ] = new WordCluePair( word, clue );
	this.numUnplacedWords ++ ;
	return true ;
  }
  
  function Puzzle_removeWord( index )
  {
     var unplacedWord = this.unplacedWords[ index ];
     this.numUnplacedWords -- ;
     if ( this.numUnplacedWords > 0 ) {
         this.unplacedWords[ index ] = this.unplacedWords[ this.numUnplacedWords ] ;
     }
     return unplacedWord ;
  }

      

  function Puzzle_placeWords()
  {
	if ( this.placedWords.length == 0 ) {
	      // no words placed yet.   pick one at random
             var randomIndex = Math.floor( Math.random() * this.numUnplacedWords/2 + this.numUnplacedWords / 2 )  ;
             var wordClue = this.removeWord( randomIndex );
             if ( wordClue.word.length <= puzzle.numColumns ) {
                // place word going across in center of puzzle
            	var row = Math.floor(this.numRows / 2) ;
             	var col = Math.floor( (this.numColumns - wordClue.word.length) / 2 );
             	this.placeWord( wordClue.word, wordClue.clue, col, row, ACROSS );
             }
             else {
                // place word going DOWN in center of puzzle
            	var col = Math.floor(this.numColumns / 2) ;
             	var row = Math.floor( (this.numRows - wordClue.word.length) / 2 );
             	this.placeWord( wordClue.word, wordClue.clue, col, row, DOWN );
             }
	}       
	  
	// try to place each word anywhere
	do {
		  var i;
		  var wordsPlaced = 0;
		  // try to start with one of the words in the longer half
		  var randomStart = Math.floor( Math.random() * this.numUnplacedWords/2 + this.numUnplacedWords / 2 )  ;
		  for ( i=0; i<this.numUnplacedWords; i++ ) {
		       var wordIndex = (i + randomStart) % this.numUnplacedWords ;
		       var wordClue = this.unplacedWords[i];
		       if ( this.placeWordAnywhere( wordClue.word, wordClue.clue ) ) {
		           wordsPlaced++;
		           this.removeWord( i );
		           break;
		       }
		  }
		  // as long as we place at least one new word, repeat and try again.
	} while ( wordsPlaced > 0 );
  }

  
  function trimPuzzle( puzzle ) 
  {
      console.log( 'in trimPuzzle puzzle=' + puzzle );
      var col;
      var row;
      var minRow = -1 ;
      var minCol = -1 ;
      var maxRow = -1 ;
      var maxCol = -1 ;
      
      // find the min row used
      for ( row = 0; row<puzzle.numRows && minRow<0; row++ ) {
          for ( col = 0; col<puzzle.numColumns && minRow<0; col++ ) {
              if ( !isBlank( getAnsChar( row, col ) ) ) {
                    minRow = row ;
              }
          }
      }

      // find the min col used
      for ( col = 0; col<puzzle.numColumns && minCol<0; col++ ) {
         for ( row = 0; row<puzzle.numRows && minCol<0; row++ ) {
              if ( !isBlank( getAnsChar( row, col ) ) ) {
                    minCol = col ;
              }
          }
      }

      // find the max row used
      for ( row = puzzle.numRows-1; row>=0 && maxRow<0; row-- ) {
          for ( col = puzzle.numColumns-1; col>=0 && maxRow<0; col-- ) {
              if ( !isBlank( getAnsChar( row, col ) ) ) {
                    maxRow = row ;
              }
          }
      }
              
      // find the max col used
      for ( col = puzzle.numColumns-1; col>=0 && maxCol<0; col-- ) {
          for ( row = puzzle.numRows-1; row>=0 && maxCol<0; row-- ) {
              if ( !isBlank( getAnsChar( row, col ) ) ) {
                    maxCol = col ;
              }
          }
      }

      var newPuzzle = new Puzzle( maxRow - minRow + 1, maxCol - minCol + 1, puzzle.blockSize );
      var pwNum ;
      for ( pwNum=0; pwNum < puzzle.placedWords.length; pwNum++ ) {
            var puzzleWord = puzzle.placedWords[ pwNum ];
            newPuzzle.placeWord( puzzleWord.word, puzzleWord.clue, puzzleWord.col - minCol, puzzleWord.row - minRow,  puzzleWord.direction );
      }
      return newPuzzle ;      
  }
  
  function writePuzzleAndClues( puzzle )
  {
    var html = '' ;
    html += "<table id='crosswordTableWrapper'><tr><td class='puzzleSide'>" ;
    html += writePuzzle( puzzle );
    html += "</td><td valign='top'>" ;
    html += "</td></tr></table>" ;
    return html ;
  }
 
 
  var seenTouch = false ;
  function pressKeyTouch( letter, event ) {
      seenTouch = true ;
      pressKey( letter, event );
  }
  
  function pressKeyClick( letter, event ) {
      if ( !seenTouch ) {
           pressKey( letter, event );
      }
  }
 
 
  var prevLetter = '';
  var prevPressTime = 0;
  
  function pressKey( letter, event ) {
     killEvent( event );
     var now = new Date().getTime();
     if ( letter == prevLetter && ((now - prevPressTime) < 130) ) {
         return ;
     }
     prevLetter = letter ;
     prevPressTime = now ;
     vibrate(15);
     hideMenu();
     if ( curRow >= 0 ) {
         showLetter( curCol, curRow, letter );
         var newRow = curRow ;
         var newCol = curCol ;
         if ( currentHilightedWord.direction == DOWN ) {
              newRow ++ ;
         }
         else {
              newCol ++ ;
         }
         if ( currentHilightedWord != puzzle.findWordAt( newRow, newCol, currentHilightedWord.direction ) ) {
             newRow = currentHilightedWord.row ;
             newCol = currentHilightedWord.col ;
         }
         setCurrentSquare( newRow, newCol );
         onStateChanged();
     }
  }

  function onStateChanged() {
      saveCurrentPuzzle();
      if ( isDone() ) {
        	showAllDone();
      }             
  }
    
  function backspace(event) {
      killEvent( event );
      showLetter( curCol, curRow, ' ' );
      var newRow = curRow ;
      var newCol = curCol ;
      if ( currentHilightedWord.direction == DOWN ) {
          if ( newRow == currentHilightedWord.row ) {
              newRow += currentHilightedWord.word.length - 1;
          }
          else {
               newRow --;
          }
      }
      else {
          if ( newCol == currentHilightedWord.col ) {
              newCol += currentHilightedWord.word.length - 1;
          }
          else {
              newCol --;
          }
      }
      setCurrentSquare( newRow, newCol );
  }
  
  
    function peek(event) {
      killEvent( event );
      showLetter( curCol, curRow, puzzle.answer.get( curRow, curCol ) );
      var newRow = curRow ;
      var newCol = curCol ;
      if ( currentHilightedWord.direction == DOWN ) {
          newRow ++;
          if ( newRow >= currentHilightedWord.row + currentHilightedWord.word.length ) {
              newRow = currentHilightedWord.row;
          }
      }
      else {
          newCol ++;
          if ( newCol >= currentHilightedWord.col + currentHilightedWord.word.length ) {
              newCol = currentHilightedWord.col;
          }
      }
      setCurrentSquare( newRow, newCol );
      onStateChanged();
      
  }
  
  function writeKeyboardRow( letters ) {
      var html = "" ;
      for ( var i=0; i<letters.length; i++ ) {
          var letter = letters.charAt( i );
          html += "<td colspan='6' " + 
                  "onclick='crosswordPage.pressKeyClick( \"" + letter + "\", event )'" +
                  "ontouchstart='crosswordPage.pressKeyTouch( \"" + letter + "\", event )'" +
                  " >" + letter + "</td>" ;
      }
      if ( letters.charAt( 0 ) == "A" ) {
	          html = "<td colspan='2' class='blankKey'></td>" +
	                 html +
	                 "<td colspan='2' class='blankKey'></td>" ;
	  }
	  else if ( letters.charAt( 0 ) == "Z" ) {
          html = "<td colspan='4' class='blankKey'></td>" + html +
                 "<td colspan='7' onclick='crosswordPage.peek(event)' ontouchstart='crosswordPage.peek(event)'><img src='images/eye.png' height='20px' width='30px'/></td>" +                 
                 "<td colspan='7' onclick='crosswordPage.backspace(event)' ontouchstart='crosswordPage.backspace(event)'><img src='images/delete.png' height='20px' width='30px'/></td>" ;
      }
      html = "<tr>" + html + "</tr>" ;
      return html ;
  }
  
  function showKeyboard( opacity ) {
    var html = '' ;
    html += "<table id='keyboard' style='opacity: " + opacity + "' >" ;
    html += writeKeyboardRow( "QWERTYUIOP" );
    html += writeKeyboardRow( "ASDFGHJKL" );
    html += writeKeyboardRow( "ZXCVBNM" );
    html += "</table>" ;
    keyboardArea.innerHTML = html ;
  }
  
  function showHelp() {
    var html = "<div id='helpArea'>Press any puzzle word</div>" ; 

    keyboardArea.innerHTML = html ;
  }
  
  function showAllDone() {
     keyboardArea.innerHTML = "<div id='cwCongrats'>CONGRATULATIONS!!!</div" ;
     if ( (numWrong == 0) || (numCorrect * 100 / (numCorrect+numWrong)) > 80 ) {
     	studyStack.finishActivity( CROSSWORD_CODE );
     }     
     setTimeout( function() {
       keyboardArea.innerHTML = 
                "<div id='allDoneArea'>" +
                    "<button onclick='crosswordPage.startNewPuzzle()'>Play Again</button>" +
                    (canUndo() ? "<button onclick='crosswordPage.undoSolution()'>Undo</button>" : "")+
                "</div>" ;
       }, 2000 );
  }

  
  function startNewPuzzle() {
        localStorage.removeItem( getItemName() );
        initSizes();
        loadPuzzle();
  }
  


	var punctuation = ".,|();: <>[]{}!@#$%^&*_-+=~`/?'0123456789\"";
	var txtAnswer = "Answer:";
	var txtShowAnswer = "Show Answer";
	var txtCheck = "Check" ;
     
    function removePunctuation( word ) {
       var retVal = "" ;
       var i;
       
       for ( i = 0; i< word.length; i++ ) {
          letter = word.charAt( i );
          if ( punctuation.indexOf( letter ) < 0 ) {
              retVal += letter ;
          }
       }
       return retVal ;
    }


   function initSizes() {
       showKeyboard( 0 );
       var titleDivHeight =  $( '#titleDiv' ).height() ;
       var keyboardHeight = $( '#keyboard' ).height() ;
       var paddingTop = 10;
	   width = $( window ).width();
	   height = $( window ).height() - keyboardHeight - titleDivHeight - paddingTop ;
	   blockSize = 24;
	   numRows = Math.floor( height / blockSize ) ;
	   numColumns = Math.floor( width / blockSize ) ;
	   if ( numRows > 20 && numColumns > 20 ) {
	       blockSize = 32;
     	   numRows = Math.floor( height / blockSize ) ;
	       numColumns = Math.floor( width / blockSize ) ;
	   }
   }

	function startPage() {
	   sharePath = '/crossword-' + studyStack.idOnStudyStack ;
	  
       keyboardArea.style.display = 'block';
	   
	   dataManager.markMostRecentlyUsed( studyStack );
	   
       initSizes();
	   
       loadPuzzle();

       contextMenu.setSetupFunc( setupContextMenu );
       document.onkeydown = keydown ;
       document.onkeypress = handleKeyPress ;
       
       hintOverKeyboard.style.top = 'auto';
       hintOverKeyboard.style.bottom = '2%';
       
	}


	function setupContextMenu() {
	    contextMenu.addMenuItem( "Check Answers", true, checkAnswers );
	    contextMenu.addMenuItem( "Solve Puzzle", true,  showSolution );
	    contextMenu.addMenuItem( "Restart", true, startNewPuzzle );
	}

  
    return {
          startPage: startPage,
          clickSquare: clickSquare,
          pressKey: pressKey,
          pressKeyClick: pressKeyClick,
          pressKeyTouch: pressKeyTouch,
          peek: peek,
          backspace: backspace,
          undoSolution: undoSolution,
          startNewPuzzle: startNewPuzzle,
          pressSquare: pressSquare,
          releaseSquare: releaseSquare
      } ;

})();

	  
