console.log( 'database.js' );

var ORDER_BY_DATE_CREATED = "createTime" ;
var ORDER_BY_DATE_USED    = "lastUsed DESC" ;
var ORDER_BY_NAME         = "lower(name), id" ;
var ORDER_BY_COLOR        = "color" ;


var dataManager = (function() {

	
	var database = null ;
	var defaultMaxDbSize = 1000000; 
	var VAR_MAX_DB_SIZE = "MAX_DB" ;
	var DB_VERSION =  1;
	var VAR_DB_VERSION = "DB_VER" ;
	var changeListener = null ;
	
	
	function createDatabase( callback ) {
	    console.log( 'in createDatabase' );
	    var errorHandler = function() { 
		    setTimeout( function() {callback(false);}, 0 );
	    };
	    var successHandler = function() { 
		    setTimeout( function() {callback(true);}, 0 );
	    };
	    
	    database.transaction( function (tx) {
            executeSqlTx( tx, 
            	'create table if not exists studystack (' +
	               'id integer primary key, ' + 
	               'name, description, idOnStudyStack integer, category integer, lastModOnStudyStack, dirty integer, lastUsed, createdBy integer, createTime, numCards integer, nextCardId integer, options, deleted integer not null default 0, color, finishedActivities' +
	             ')', 
	             null, 
	             function() {
	             	executeSqlTx( tx, 
	             		'create table if not exists studyfield (' +
                            'id integer primary key, ' +
                            'name, studystack_id integer, idOnStudyStack integer, dirty integer, deleted integer default 0,' +
                            'FOREIGN KEY(studystack_id) REFERENCES studystack(id)' +
                        ')', 
                        null, 
                        function() {
					    	executeSqlTx( tx, 
					    		'create table if not exists studydata  (' +
		                        	'id integer primary key,' +
		                            'value, studystack_id integer, position integer, studycard_id integer, studyfield_id integer, cardIdOnStudyStack integer, dirty integer, deleted integer default 0, ' +
		                            'FOREIGN KEY(studystack_id) REFERENCES studystack(id),' +
		                            'FOREIGN KEY(studyfield_id) REFERENCES studyfield(id)' +
		                        ')', 
		                        null, 
		                        
		                        function() {
							    	executeSqlTx( tx, 
							    		'create table if not exists categories  (' +
				                        	'id integer primary key, name, url' +
				                        ')', 
				                        null, 
				                        successHandler, 
				                        errorHandler );
		                        },		                        
		                        errorHandler );
                        }, 
                        errorHandler );
	             }, 
	             errorHandler );
	       } );
	    window.localStorage.setItem( VAR_DB_VERSION, DB_VERSION );
	}
	
	function setChangeListener( newListener ) {
	    changeListener = newListener ;
	}
	 
	
	
	function initializeDatabase( complete ) {
	    var maxDbSize = window.localStorage.getItem( VAR_MAX_DB_SIZE ); 
	    if ( !maxDbSize ) {
	        maxDbSize = defaultMaxDbSize ;
	    }
	    console.log( 'initializeDatabase maxDbSize=' + maxDbSize );
	    database = window.openDatabase( 'studystackdb38', '1.0', 'StudyStack Database', maxDbSize );
        createDatabase( complete );
	}
	
	 
	function insertStudyStack( name, idOnStudyStack, category, lastModOnStudyStack, createdBy, createTime, numCards, nextCardId, options, color, callback ) {
	
	    database.transaction( function (tx) {
	     	 var sql = "insert into studystack (name, idOnStudyStack, category, lastModOnStudyStack, createdBy, createTime, numCards, nextCardId, options," + 
	     	 									"dirty, deleted, color, finishedActivities ) " + 
	     	                           "values (?,?,?,?,?,?,?,?,?,0,0,?,'')" ;
	     	 var values = [ name, idOnStudyStack, category, lastModOnStudyStack, createdBy, createTime, numCards, nextCardId, options, color ];
	     	 executeSqlTx( tx, sql, values,  
	     	       function (tx, results ) {
	     	           var newStackId = results.insertId ;
	     	           console.log( 'insertStudyStack new id = ' + newStackId );
	     	           if ( changeListener ) {
	     	               setTimeout( function() { changeListener.stackAdded( newStackId, name ); }, 100 );
	     	           }
	     	           setTimeout( function() { callback( newStackId ); }, 0 );
	     	       },
					function ( tx, error ) {
					    console.error( 'insertStudyStack SqlErrorHandler error.code=' + error.code + ' error.message=' + error.message + 'tx=' + tx + ' error=' + error );
 	     	            setTimeout( function() { callback( -1 ); }, 0 );
					}
              );
	     });
	}

	
	
	var mostRecentlyUsedId = -1;
	
	function markMostRecentlyUsed( studyStack ) {
	    if ( studyStack.id != mostRecentlyUsedId ) {
	
	       database.transaction(function (tx) {
	          var values = [ studyStack.id ];
	          executeSqlTx( tx,"update studystack set lastUsed=datetime('now') where id=?", 
	              values, 
	              function (tx, results) {
	                  mostRecentlyUsedId = studyStack.id ;
	              }, 
	              null );
	       });
	    }
	}
	
	
	function markForDeletion( studyStackId, studyCardId, callback ) {
	       database.transaction(function (tx) {
	          var values = [ studyStackId, studyCardId ];
	          executeSqlTx( tx,"update studydata set deleted=1,dirty=1 where studystack_id=? and studycard_id=?", 
	              values,
	              callback, 
	              null );
	       });
	}
	
	function markSetForDeletion( studyStackId, callback ) {
	      database.transaction(function (tx) {
	          var values = [ studyStackId ];
	          executeSqlTx( tx,"update studystack set deleted=1 where id=?", 
	              values,
	              callback, 
	              callback );
	       });
	}
	
	function markFieldForDeletion( studyStackId, fieldId, callback ) {
      database.transaction(function (tx) {
          var values = [ studyStackId, fieldId ];
          executeSqlTx( tx,"update studyfield set deleted=1 where studystack_id=? AND id=?", 
              values,
              callback, 
              callback );
       });
	}
	
/*	
	function deleteCard( studyStackId, studyCardId ) {
	
	       database.transaction(function (tx) {
	          var values = [ studyStackId, studyCardId ];
	          executeSqlTx( tx,"delete from studydata where studystack_id=? and studycard_id=?", 
	              values,
	              genericSqlSuccessHandler, 
	              null );
	       });
	}
*/	
	
	
	
	function getRecentStacks( limit, callback ) {
	    database.transaction(function (tx) {
	          executeSqlTx( tx,"SELECT * FROM studystack WHERE lastUsed <> '' and deleted = 0 order by lastUsed DESC limit " + limit, [], function (tx, results) {
	              var len = results.rows.length, i;
	              for (i = 0; i < len; i++){
	                  callback( makeStackFromResultSet( results.rows.item(i) ) );
	              }
	              if ( len == 0 ) {
	             	 callback( createStackFromJson( SAMPLE_SET ) );
	              }
	          }, null );
	     });
	}
	
	
	function getStackList( sortOrder, callback, completeFunc ) {
	    if ( sortOrder == null ) {
	        sortOrder = ORDER_BY_NAME ;
	    }
	    database.transaction(function (tx) {
	          executeSqlTx( tx,'SELECT * FROM studystack WHERE deleted = 0 ' + sortOrder, [], function (tx, results) {
	              var len = results.rows.length, i;
	              for (i = 0; i < len; i++) {
	                  var stack = makeStackFromResultSet( results.rows.item(i) );
	                  if ( stack.name.trim().length > 0 || stack.numCards > 0 ) { 
	                  	  setTimeout( callback, 1, stack );
	                  }
	              }
	              if ( completeFunc ) {
	                  setTimeout( completeFunc, 0 );
	              }
	          }, null );
	     });
	}
	
	
	
	function getStack( stackId, callback ) {
		if ( stackId == SAMPLE_SET_ID ) {
			callback( createStackFromJson( SAMPLE_SET ) );
		}
	    getStackByField( "id", stackId, true, callback );
	}
	
	
	function getStackWithServerId( serverId, callback ) {
	   getStackByField( "idOnStudyStack", serverId, true, callback );
	}

	function getStackInfo( stackId, callback ) {
	    getStackByField( "id", stackId, false, callback );
	}
	
	function getDirtyStack( callback ) {
        getStackWhere( "dirty=1 and category>0 and ((createdBy<=0) or (createdBy=" + userId + "))", true, true, callback ); 
	}
	
	
	function getStackByField( fieldName, idValue, includeData, callback ) {
        getStackWhere( fieldName + "= '" + idValue + "'", includeData, false, callback ); 
    }
    
    function getStackWhere( whereClause, includeData, includeDeleted, callback ) {
        var sql = "SELECT * FROM studystack WHERE " + whereClause + " limit 1";        
	    database.transaction( function (tx) {
	          executeSqlTx( tx,sql, [], 
	          		function (tx, results) {
	               		if ( results.rows.length == 1 ) {
	               		    // we found the stack
	               		    var stack = makeStackFromResultSet( results.rows.item(0) );
	               		    if ( includeData ) {
	               		    	getStackFields( stack, function() {
		               		        getDataItems( stack, includeDeleted, function() {
		               		            stack.updateNumCards();
	               		        		setTimeout( callback, 1, stack );
	               		        	} );
	               		    	} );
	               		    }
	               		    else {
           		        		setTimeout( callback, 1, stack );
	               		    } 
	               	    }
	               	    else {
	               	        console.warn( 'getStack did not find stack where: ' + whereClause );
	               	        setTimeout( callback, 1, stack );
	               	    }
	          		}, 
	          		function (tx, error) {
	          			console.error( 'getStack for ' + whereClause + ' error tx=' + tx + ' error=' + error );
	                    console.error( 'getStack error.code=' + error.code + ' error.message=' + error.message );
               	        setTimeout( callback, 1, null );
	          		}
	          );
	     });
	}      
		
	function getDataItems( studystack, includeDeleted, callback ) {
	    database.transaction( function (tx) {
	    	var delSql = includeDeleted ? '' : ' and deleted = 0 ';
	          executeSqlTx( tx,'SELECT * FROM studydata WHERE studystack_id=?' + delSql + ' order by position, id, studyfield_id', [ studystack.id ], 
	          		function (tx, results) {
	          		    for ( var r=0; r<results.rows.length; r++ ) {
	          		        var resultSet = results.rows.item(r); 
	          		        var dataElement =  makeDataElementFromResultSet( resultSet );
	          		        var cardId = parseFloat( resultSet.studycard_id );
	          		        var fieldId = resultSet.studyfield_id ;
	          		        var row = studystack.getRowByCardId( cardId );
	          		        row.addDataElement( fieldId, dataElement )
	               	    }
	               	    setTimeout( callback, 1 );
	          		}, 
	          		function (tx, error) {
	          			console.error( 'getDataItems error tx=' + tx + ' error=' + error );
	                    console.error( 'getDataItems error.code=' + error.code + ' error.message=' + error.message );
	               	    setTimeout( callback, 1 );
	          		}
	          );
	     });
	}
	
	
	function getStackFields( studystack, callback ) {
	    database.transaction( function (tx) {
	          executeSqlTx( tx,'SELECT * FROM studyfield WHERE studystack_id=? AND deleted = 0 order by id', [ studystack.id ], 
	          		function (tx, results) {
	          		    for ( var r=0; r<results.rows.length; r++ ) { 
	          		        var field =  makeStudyFieldFromResultSet( results.rows.item(r) );
	          		        studystack.addField( field );
	               	    }
	               	    setTimeout( callback, 1 );
	          		}, 
	          		function (tx, error) {
	          			console.error( 'getStackFields error tx=' + tx + ' error=' + error );
	                    console.error( 'getStackFields error.code=' + error.code + ' error.message=' + error.message );
	               	    setTimeout( callback, 1 );
	          		}
	          );
	     });
	}
	
	
	
	
	
	
	function getNewStack() {
	    var deferred = $.Deferred();
	    database.transaction( function (tx) {
	          // see if there is an existing stack that has no cards in it.
	          executeSqlTx( tx,'SELECT * FROM studystack WHERE numCards=0 AND nextCardId=1  order by createTime DESC limit 1', [], 
	          		function (tx, results) {
	               		if ( results.rows.length == 1 ) {
	               		    // we found a stack that has no cards in it.   So lets have the user update 
	               		    // that set
	               		    var stack = makeStackFromResultSet( results.rows.item(0) );
	               		    getStackFields( stack, function() {
	               		        getDataItems( stack, false, function() {
	                     		    deferred.resolve( stack ); 
	               		        } );
	               		    } ); 
	               	    }
	               	    else {
	               	        // no empty sets, so lets create a new one and have the user edit that one
	               	        var promise = createNewStudyStack();
	               	        promise.done( function( stack ) {
	               	           deferred.resolve( stack );
	               	        } );
	               	    }
	          		}, 
	          		function (tx, error) {
	          			console.error( 'getNewStack error tx=' + tx + ' error=' + error );
	                    console.error( 'getNewStackError error.code=' + error.code + ' error.message=' + error.message );
	                    deferred.reject( null );
	          		}
	          );
	     });
	     return deferred.promise();
	}
	
	
	 
	var defaultField1Name = "Question" ;
	var defaultField2Name = "Answer" ;
	
	
	function createNewStudyStack() {
	    var deferred = $.Deferred();
	    
	    database.transaction( function (tx) {
	     	 var studyStack = getNewStudyStack();
	     	 var values = [ studyStack.name, studyStack.idOnStudyStack, studyStack.category, studyStack.lastModOnStudyStack, userId ];
	     	 executeSqlTx( tx, "insert into studystack " + 
	     	                "(name, idOnStudyStack, category, lastModOnStudyStack, dirty, lastUsed, createdBy, createTime, numCards, nextCardId, options, deleted, finishedActivities) " + 
	     	                "values (?,?,?,?,1,null,?,datetime('now'),0,1,'',0,'')", 
	                values,  
	          		function (tx, results) {
	               		studyStack.id = results.insertId ;
	               		executeSqlTx( tx, "insert into studyfield ( name, studystack_id, idOnStudyStack, dirty) values ( ?, ?, -1, 1 )",
	               		       [ defaultField1Name, studyStack.id ],
	               		       function ( tx, results ) {
	               		            studyStack.addField( new StudyField( results.insertId, defaultField1Name, -1, 1 ) );
				               		executeSqlTx( tx, "insert into studyfield ( name, studystack_id, idOnStudyStack, dirty) values ( ?, ?, -1, 1 )",
				               		       [ defaultField2Name, studyStack.id ],
				               		       function ( tx, results ) {
	                         		            studyStack.addField( new StudyField( results.insertId, defaultField2Name, -1, 1 ) );
						                 		deferred.resolve( studyStack );
				               		       },
				               		       function ( tx, error ) {
				               		            console.error( 'insert into studyfield error' );
				                         		deferred.resolve( studyStack );
				               		       }
				               		);
	               		       
	               		       },
	               		       function ( tx, error ) {
	               		            console.error( 'insert into studyfield error' );
	                         		deferred.resolve( studyStack );
	               		       }
	               		);
	          		}, 
	          		function (tx, error) {
	          		    console.error( "insert into studystack failed " + error );
                   dumpProps( error );
	          		    deferred.reject( null );
	          		}
	          );
	     });
	     
	     return deferred.promise();
	}
	
	
	function clearStacksDirtyFlags( stackId ) {
	    database.transaction( function (tx) {
	     	 var values = [ stackId ];
	     	 executeSqlTx( tx, "update studystack set dirty=0 where id=?", values,  genericSqlSuccessHandler, genericSqlErrorHandler );
	     	 executeSqlTx( tx, "update studyfield set dirty=0 where studystack_id=?", values,  genericSqlSuccessHandler, genericSqlErrorHandler );
	     	 executeSqlTx( tx, "update studydata  set dirty=0 where studystack_id=?", values,  genericSqlSuccessHandler, genericSqlErrorHandler );
	     });
	}
	
	function updateField( tableName, fieldName, fieldValue, rowId, callback ) {
	    database.transaction( function (tx) {
	         var sql;
	         if ( fieldName != 'dirty' ) {
	     	      sql = "update " + tableName + " set " + fieldName + "=?, dirty=1 where id = ?" ;
	     	 }
	     	 else {
	     	      sql = "update " + tableName + " set " + fieldName + "=? where id = ?" ;
	     	 }
	     	 var values = [ fieldValue, rowId ];
	     	 executeSqlTx( tx, sql, values,  (callback ? callback : genericSqlSuccessHandler), callback );
	     	 
	     	  if ( tableName=='studystack' && fieldName=='numCards' && changeListener != null ) {
     	           if ( changeListener ) {
     	                changeListener.cardAdded( rowId, fieldValue );
     	           }
	     	  } 
	     	 
	     	 
	     });
	}
	
	
	function insertStudyField( studyStackId, studyField, callback ) {
	    database.transaction( function (tx) {
	     	 var sql = "insert into studyfield (name, studystack_id, idOnStudyStack, dirty ) values (?,?,?,?)" ;
	     	 var values = [ studyField.name, studyStackId, studyField.idOnStudyStack, studyField.dirty ];
	     	 executeSqlTx( tx, sql, values,  
	     	       function (tx, results ) {
	     	           studyField.id = results.insertId ;
	               	   setTimeout( callback, 1 );
	     	       },
	     	       null );
	     });
	}
	
	function updateStudyField( studyField, callback ) {
	    database.transaction( function (tx) {
	     	 var sql = "update studyfield set name=?, idOnStudyStack=?, dirty=? where id=?" ;
	     	 var values = [ studyField.name, studyField.idOnStudyStack, studyField.dirty, studyField.id ];
	     	 executeSqlTx( tx, sql, values,  
	     	       function (tx, error ) {
  	               	    setTimeout( callback, 1 );
	     	       },
	     	       null );
	     });
	}
	
	function saveStudyField( studyStackId, studyField, callback ) {
	     if ( studyField.id < 0 ) {
	         insertStudyField( studyStackId, studyField, callback );
	     }
	     else {
	         updateStudyField( studyField, callback ) ;
	     }
	}
	
	
	
	function saveDataElement( studyStackId, cardId, fieldId, dataElement, callback ) {
	    if ( dataElement.id < 0 ) {
	        insertDataElement( studyStackId, cardId, fieldId, dataElement, callback ) ;
	    }
	    else {
	        updateDataElement( dataElement, callback );
	    }
    }
	
	
	
	
	function insertDataElement( studyStackId, cardId, fieldId, dataElement, callback ) {
	
	    database.transaction( function (tx) {
	     	 var sql = "insert into studydata (value, studystack_id, position, studycard_id, studyfield_id, cardIdOnStudyStack, dirty, deleted) values (?,?,?,?,?,?,?,0)" ;
	     	 var values = [ dataElement.value, studyStackId, dataElement.position, cardId, fieldId, dataElement.cardIdOnStudyStack, dataElement.dirty ];
	     	 executeSqlTx( tx, sql, values,  
	     	       function (tx, results ) {
	     	           dataElement.id = results.insertId ;
	     	           setTimeout( callback, 1, dataElement );
	     	       },
	     	       null );
	     });
	
	}
	
	function updateDataElement( dataElement, callback ) {
	
	    database.transaction( function (tx) {
	     	 var sql = "update studydata set value=?, position=?, dirty=? where id=?" ;
	     	 var values = [ dataElement.value, dataElement.position, dataElement.dirty, dataElement.id ];
	     	 executeSqlTx( tx, sql, values,  
	     	       function (tx, error ) {
	     	           setTimeout( callback, 1, dataElement );
	     	       },
	     	       null );
	     });
	}
	
	
	
	function updateStudyStack( studyStack, callback ) {
	    database.transaction( function (tx) {
	     	 var values = [ studyStack.name, studyStack.description, studyStack.idOnStudyStack, studyStack.category, studyStack.lastModOnStudyStack, 
	     	                studyStack.dirty, studyStack.numCards, studyStack.nextCardId, studyStack.options, studyStack.id ];
	     	 executeSqlTx( tx, "update studystack set name=?, description=?, idOnStudyStack=?, category=?, lastModOnStudyStack=?, dirty=?, numCards=?, nextCardId=?, options=? where id = ?", 
	                values,  
	          		function (tx, results) {
	     	            setTimeout( callback, 1, studyStack );
	          		}, 
	          		function (tx, error) {
	     	            setTimeout( callback, 1, null );
	          		}
	          );
	     });
	}
	
	
	function getLastModifiedStackList( callback ) {
	    database.transaction(function (tx) {
	          executeSqlTx( tx,'SELECT idOnStudyStack, lastModOnStudyStack FROM studystack WHERE deleted=0 AND idOnStudyStack>0', [], function (tx, results) {
	              var idList = '' ;
	              var modTimeList = '' ;
	              var i;
	              var lastItem  = results.rows.length-1;
	              for (i = 0; i <= lastItem; i++){
	                  var resultSet = results.rows.item(i) ;
	                  idList += resultSet.idOnStudyStack ;
	                  modTimeList += resultSet.lastModOnStudyStack ;
	                  if ( i != lastItem ) {
	                      idList += ',';
	                      modTimeList += ',';
	                  } 
	              }
	              setTimeout( function() { callback( idList, modTimeList ); }, 0 );
	          }, null );
	     });
	}
	
	function updateIdOnStudyStack( keyType, localId, idOnStudyStack, callback ) {
	
		var tableName = null ;
		var columnName = null ;
		   
		if ( keyType == 's' ) {
		   	tableName = 'studystack' ;
		   	columnName = 'idOnStudyStack' ;
		}
		else if ( keyType == 'f' ) {
			tableName = 'studyfield' ;
		   	columnName = 'idOnStudyStack' ;
		}
		else if ( keyType == 'c' ) {
			tableName = 'studydata';
			columnName = 'cardIdOnStudyStack';
		}
		else {
		    alert( 'invalid keyType: ' + keyType + '.' );
		}
		
		var sql = "update " + tableName + " set " + columnName + "=? where id=? and " + columnName + "=-1" ;
		database.transaction(function (tx) {
		    var values = [ idOnStudyStack, localId ];
		    executeSqlTx( tx, sql, values, callback, callback  );
		});
	}

	var dbstate = 0;
	
	function executeSqlTx( tx, sqlStatement, arguments, callback, errorCallback ) {
		tx.executeSql( sqlStatement, arguments, 
			callback, 
			function( tx, error ) {
			genericSqlErrorHandler( tx, error ); 
		    console.error( 'databaseManager.executeSql ' + sqlStatement );
		    if ( arguments != null ) {
		        for ( var a=0; a<arguments.length; a++ ) {
		             console.log( '     argument[' + a + '] = ' + arguments[a] );
		        }
		    }
		    if ( errorCallback ) {
				errorCallback()
			} 
		} );
	}
	
	function genericSqlErrorHandler( tx, error ) {
	    console.error( 'genericSqlErrorHandler ------------------------' );
	    console.error( ' tx=' + getProps( tx ) );
	    console.error( ' error=' + getProps( error ) );
	}
	
	function genericSqlSuccessHandler( tx, resultSet ) {
	}
	
	
	function setLocalItem( name, value ) {
//	     console.log( 'setLocalItem for '+ name + ' saving ' + value );
	     window.localStorage.setItem( name, value );
	}
	
	function getLocalItem( name, defaultValue ) {
	     var value = window.localStorage.getItem( name );
	     if ( value == null ) {
	         value = defaultValue ;
	     }
//	     console.log( 'getLocalItem for '+ name + ' returning ' + value );
	     return value ;
	}


	

    return {
        clearStacksDirtyFlags: clearStacksDirtyFlags,
        getDirtyStack: getDirtyStack,
        getLastModifiedStackList: getLastModifiedStackList,
    	getLocalItem: getLocalItem,
    	getNewStack: getNewStack,
    	getRecentStacks: getRecentStacks,
    	getStack: getStack,
    	getStackInfo: getStackInfo,
    	getStackList: getStackList,
    	getStackWithServerId: getStackWithServerId,
    	initializeDatabase:  initializeDatabase,
    	insertDataElement: insertDataElement,
    	insertStudyStack: insertStudyStack,
    	insertStudyField: insertStudyField,
    	markFieldForDeletion: markFieldForDeletion,
    	markForDeletion: markForDeletion,
    	markMostRecentlyUsed: markMostRecentlyUsed,
    	markSetForDeletion: markSetForDeletion,
    	saveDataElement: saveDataElement,
    	saveStudyField: saveStudyField,
    	setChangeListener: setChangeListener,
    	setLocalItem: setLocalItem,
    	updateDataElement: updateDataElement,
    	updateField: updateField,
    	updateIdOnStudyStack: updateIdOnStudyStack,
    	updateStudyField: updateStudyField,
    	updateStudyStack: updateStudyStack,
    } ;

})();

