console.log( 'flashcard.js' );

var flashcardPage = (function() {

  var emSizesForLength;
  var MIN_EM  ;
  var currentCardIndex ;
  var currentSide ;
  var reverseOrder ;
  var retries ;
  var pileNames ; 		 
  var cardIndexes ;
  var ANIMATE_SPEED ;
  var autoPlay ;
  var readAloud  ;
  var speechRate = dataManager.getLocalItem( '', 1.0 );

  var currentCard = $( '#fc_currentCard' );
  var curSides = [];
  var remainingCards = $( '#remainingCards' );
  var incorrectCards = $( '#incorrectCards' );
  var correctCards = $( '#correctCards' );
  var passCompleteDiv = $( '#passComplete' );

  addTapListener( passCompleteDiv, function() {
      var numIncorrect = getCount( 'incorrect' );
      if ( numIncorrect > 0 ) {
    	  retryIncorrect();
      }
      else {
    	  restartAll();
      }
  }) ;
  
   addTapListener( currentCard, function() {
       if ( currentCardIndex == -1 ) {
       	    if ( getCount( 'incorrect' ) > 0 ) {
		         retryIncorrect();
		    }
		    else {
		    	 restartAll();
		    }
       }
       else {
       	   flipCard();
       }
   } );
   
   var box1 = $( '#incorrectBox' );
   addTapListener( box1, function() {
     moveCurrentCardToPile( incorrectCards, getNextCard );
   } );

   var box2 = $( '#remainingBox' );
   addTapListener( box2, function() {
     moveCurrentCardToPile( remainingCards, getNextCard );
   } );

   var box3 = $( '#correctBox' );
   addTapListener( box3, function() {
     moveCurrentCardToPile( correctCards, getNextCard );
   } );


   var pile1 = incorrectCards;
   addTapListener( pile1, function() {
      moveToCenter( 'incorrect' );
      setTimeout( flashcardPage.saveBoxStates, 1000 );
   } );

   var pile2 = remainingCards;
   addTapListener( pile2, function() {
      moveCurrentCardToPile( remainingCards, getNextCard );
   } );

   var pile3 = correctCards;
   addTapListener( pile3, function() {
      moveToCenter( 'correct' );
      setTimeout( flashcardPage.saveBoxStates, 1000 );
   } );

  

    
  function flipCard() {
	   console.log( 'flip' );
	   cancelSpeaking();
	   if ( studyStack != null ) {
		   currentSide = ( currentSide + 1 ) % studyStack.getNumSides() ;
		   animateFlip();
	   }
   }


   var degrees = 0 ;
   var fcCardFlipper = null ;
   var frontSide = 0;
   var backSide = 1;
   
   function animateFlip() {
       initCardHtml( curSides[backSide], currentCardIndex, currentSide );   
       degrees += 180;
       console.log( 'degrees=' + degrees );
       fcCardFlipper.style.webkitTransform='rotateX(' + degrees + 'deg)';
       if ( frontSide == 0 ) {
           frontSide = 1;
           backSide = 0;
       }
       else {
           frontSide = 0;
           backSide = 1;
       }
       
       sayCurrentCard( currentCardIndex, currentSide );
       
   }

   
   function clearCurrentCardHeight() {
        setTimeout( function() {
   	    	currentCard.css( 'height', '' );
   	    }, 1 );
   }

   
   function initCurrentCardHtml( cardIndex, topSide ) {
       clearPassComplete() ;   
       var nextSide = (topSide + 1 ) % studyStack.getNumSides();
       console.log( 'initCurrentCardHtml cardIndex=' + cardIndex + ' topSide=' + topSide + ' nextSide=' + nextSide + ' numSides=' + studyStack.getNumSides() + ' degrees=' + degrees );
       initCardHtml( curSides[frontSide], cardIndex, topSide );
       initCardHtml( curSides[backSide], cardIndex, nextSide );
       fcCardFlipper.style.webkitTransform='rotateX(' + degrees + 'deg)';
       currentCard.css( 'display', 'block' );
       currentCard.css({ opacity: 1.0 });
       
       sayCurrentCard( cardIndex, topSide );
   }
   
   
   function toggleSpeech() {
	   readAloud = !readAloud ;
       dataManager.setLocalItem( 'fcReadAloud', readAloud ? 'Y' : 'N' ) ;
	   if ( readAloud ) {
		   sayCurrentCard( currentCardIndex, currentSide );
	   }
   }
   
   var speechId = '' ;
   
   function sayCurrentCard( cardIndex, side ) {
	   speechId = cardIndex + '.' + side ;
	   var curSpeechId = speechId ;
       var vol = (side==0) ? 1.0 : 0.6;
       if ( reverseOrder ) {
           side = studyStack.getNumSides() - side - 1;
       }
       var text = studyStack.getCardSide( cardIndex, side );
       speakTextVol( text, vol, function() {
    	    if ( autoPlay && (speechId == curSpeechId) ) {
    	    	setTimeout( autoStep, 1000 );
    	    }
       } );
   }   

   
   

   function speak(text) {
       speakTextVol( text, 1.0 );
   }

   function speakHelp(text) {
       speakTextVol( text, 0.5 );
   }

   function isSpeechSupported() {
	   return (typeof SpeechSynthesisUtterance !== 'undefined') ||
	          (typeof TTS !== 'undefined') ;
   }
   
   function speakTextVol(text, vol, onDoneFunc) {
       if ( readAloud ) {
    	   if (typeof SpeechSynthesisUtterance !== 'undefined') {
    		   speakWithSpeechSynthesisUtterance( text, vol, onDoneFunc ) ;
    	   }
    	   else if (typeof TTS !== 'undefined') {
    		   speakWithTTS( text, vol, onDoneFunc );
    	   }
       } 
   }
   
   function speakWithSpeechSynthesisUtterance( text, vol, onDoneFunc ) {
       var u = new SpeechSynthesisUtterance();
       u.text = text;
       u.volume = vol;
       u.lang = 'en-US';
       u.rate = speechRate ;
       u.onend = function (e) {
    	   console.log( 'onend');
    	   console.log( getProps( e ) );
    	   if ( onDoneFunc ) {
    		   setTimeout( onDoneFunc, 1 );
    	   }
       };
       u.onerror = function (e) {
    	   console.log( 'onerror ' + e );
       };

       speechSynthesis.speak(u);
   }
   
   function speakWithTTS( text, vol, onDoneFunc ) {
	   TTS.speak({
           text: text,
           rate: 1.0
       }, function () {
    	   if ( onDoneFunc ) {
    		   setTimeout( onDoneFunc, 1 );
    	   }
       }, function (e) {
    	   console.log( 'onerror ' + e );
       });
   }

   function speakPause() {
       if ( readAloud ) {
    	   if ( typeof SpeechSynthesisUtterance !== 'undefined' ) {
	           var u = new SpeechSynthesisUtterance();
	           u.text = ' ';
	           u.lang = 'en-US';
	           u.volume = 0.0;
	           u.onend = function () { };
	           u.onerror = function (e) { };
	
	           speechSynthesis.speak(u);
           }
       	   else {
		        speakWithTTS( ' ', 0, null );
	       }
       }
   }

   function cancelSpeaking() {
	   console.log( 'cancelSpeaking' );
       if ( readAloud ) {
    	   if ( typeof SpeechSynthesisUtterance !== 'undefined' ) {
               speechSynthesis.cancel();
    	   }
    	   else {
    		   speakWithTTS( '', 0, null );
    	   }
       }
   }

   function initPileTopWithCard( pile, cardNumber ) {
       console.log( 'initPileTopWithCard cardNumber=' + cardNumber + ' pile.id=' + pile.attr( 'id' ) );
       initCardHtml( pile, cardNumber, 0 );
   }
   
   
  

   function initCardHtml( card, cardIndex, side ) {
        if ( cardIndex < 0 ) {
            return ;
        }
        if ( reverseOrder ) {
             side = studyStack.getNumSides() - side - 1;
        }
        var text = studyStack.getCardSide( cardIndex, side );
        console.log( 'intCardHtml text=' + text );
        var fieldName = '' ;
        if ( studyStack && studyStack.fields && studyStack.fields.length > side && studyStack.fields[side].name ) {
        	fieldName = escapeHtml(studyStack.fields[side].name) ;
        }
        card.html( '<h2 class="fc_cardTitle">' + fieldName + '</h2>' 
                      + '<div id="fc_innerDiv" style="font-size: ' + getAppropriateFontSize( text )+ '">' 
                         + escapeHtml(text)
                      + '</div>' 
                   );
         card.css( 'backgroundColor', (side==0) ? colorCodes[ studyStack.color ] : otherSideColorCodes[ studyStack.color ]  );
         var className =  (card==currentCard) ? 'linedCard' : 'linedCardSmall' ;
         if ( side == 0 ) {
              card.addClass( className );
         }
         else {
              card.removeClass( className );
         }
         
         if ( studyStack.id == SAMPLE_SET_ID && cardIndex == currentCardIndex ) {
        	 setTimeout( function() {
     	         card.append( "<div class='tapToFlip'>Tap to flip</div>" );
        	 }, 500);
        }

   }
  
  
  
  function getAppropriateFontSize( cardText ) {
        var length = cardText.length;
        if ( length < emSizesForLength.length ) {
            return emSizesForLength[ length ] + "em" ;
        }
        length -= emSizesForLength.length ;
        var em = emSizesForLength[ emSizesForLength.length - 1 ];
        em -= (MIN_EM * length / 256) ;
        if (window.innerHeight < window.innerWidth) {
           // landscape
           em *= 0.8;
        }        
        if ( em < MIN_EM ) {
            em = MIN_EM ;
        }
        return em + "em";
   }

   function getPileLabel( pileName ) {
	   if ( pileName == 'correct' ) {
		   return "Know";
	   }
	   if ( pileName == 'incorrect' ) {
		   return "Don't know" ;
	   }
	   return "Remaining" ;
   }
   function setCount( pileName, newCount, hideIfZero ) {
       if ( newCount != 0 && (!cardIndexes || !cardIndexes[ pileName ]) ) {
           console.log( '******   setCount called too soon ****** pileName=' + pileName );
           return ;
       }
       
       var counter = document.getElementById( pileName + 'CardCount' );
       var numCardsObj = document.getElementById( pileName + 'Cards' );
       console.log( '>>>>>>  setCount pileName=' + pileName + ' newCount=' + newCount );
       counter.innerHTML = getPileLabel( pileName ) + " (" + newCount + ")";
       cardIndexes[ pileName ].length = newCount ;
       if ( newCount == 0 ) {
            if ( hideIfZero ) {
                console.log( 'newCount is zero  hiding numCardsObj for ' + pileName );
                numCardsObj.style.visibility = 'hidden';
            }
       }
       else {
            var cardNumber = cardIndexes[ pileName ][ newCount-1 ];
            console.log( 'pileName=' + pileName +  '  top count=' + (newCount-1) + ' cardNumber=' + cardNumber );
            initPileTopWithCard( $( '#' + pileName + "Cards" ), cardNumber ) ;
            numCardsObj.style.visibility = 'visible';
       }
       console.log( 'setCount pileName=' + pileName + ' newCount=' + newCount + ' isZero=' + (newCount==0) );
    }


   function adjustCount( pileName, adjustment ) {
       var newCount ;
       if ( adjustment == 0 ) {
            newCount = 0;
       }
       else {
            newCount = getCount( pileName ) + adjustment ;
       }
       setCount( pileName, newCount, true );
   }

   function getCount( pileName ) {
       var counter = document.getElementById( pileName + 'CardCount' );
       var pile = document.getElementById( pileName + 'Cards' );
       if ( pile == null ) {
            return 0;
       }
       var text = counter.innerHTML ;
       text = text.substr( text.indexOf( '(' ) + 1 );
       return parseInt( text );
   }



	function startPage() {
		currentCard = $( '#fc_currentCard' );
		remainingCards = $( '#remainingCards' );
		incorrectCards = $( '#incorrectCards' );
		correctCards = $( '#correctCards' );
	   
       frontSide = 0;
       backSide = 1;
       emSizesForLength = [ 0, 10, 8, 6, 4, 3, 2.6, 2.2, 1.9, 1.7 ];
       MIN_EM = 0.7 ;
       currentCardIndex = -1;
       currentSide = 0;
       reverseOrder = false ;
       autoPlay = dataManager.getLocalItem( 'fcAutoPlay' ) == 'Y' ;
       readAloud = dataManager.getLocalItem( 'fcReadAloud' ) == 'Y' ;
       retries = 0;
       pileNames = [ 'incorrect', 'remaining', 'correct' ]; 		 
       cardIndexes = null;
       ANIMATE_SPEED = 1;
	
       currentCard.css({ opacity: 0.0 });  
	   prepareFlipper() ;

       
	   infoCard = $( '#infoCard' );
	   infoCardContent = $( '#infoCardContent' );	   
	   
	   sharePath = '/flashcards-' + studyStack.idOnStudyStack ;
	   console.log( 'flashcardPage.startPage studyStack.id = ' + studyStack.id );
	   console.log( 'flashcardPage.startPage studyStack.name = ' + studyStack.name );
	   dataManager.markMostRecentlyUsed( studyStack );
	   //refreshTitle();
	   
	
       contextMenu.setSetupFunc( setupContextMenu );
       
       cardIndexes = new Array();
       cardIndexes[ 'correct' ] = createInitialOrderArray( 0 ) ;
       cardIndexes[ 'incorrect' ] = createInitialOrderArray( 0 ) ;
       cardIndexes[ 'remaining' ] = createInitialOrderArray( 0 ) ;
       
       setCount( 'remaining', 0, true );
       setCount( 'correct', 0, true);
       setCount( 'incorrect', 0, true);
       
       
       startOver();
	}


	function endPage() {
        if ( autoPlayTimer != null ) {
            clearTimeout( autoPlayTimer );
            autoPlayTimer = null ;
        }
        autoPlay = false ;
	}
	
	
    function moveCurrentCardToPile( pile, thenDoFunc ) {
    	 cancelSpeaking();
         clearPassComplete() ;
	     if ( currentCardIndex >= 0 ) {
	         moveCard( currentCard, pile, thenDoFunc );
	     }
         setTimeout( flashcardPage.saveBoxStates, 500 );
    }

	function setupContextMenu() {
	    contextMenu.addMenuItem( "Shuffle", true, shuffleCards );
	    contextMenu.addMenuItem( "Retry Incorrect", (getCount( 'incorrect' ) > 0),  retryIncorrect );
	    contextMenu.addMenuItem( "Restart All", true, restartAll );
	    var otherSideIndex = reverseOrder ?  0 :  (studyStack.getNumSides()-1 );
	    contextMenu.addMenuItem( "Show " + studyStack.fields[ otherSideIndex ].name + " first", true, reverseSides );
	    contextMenu.addMenuItem( "Turn auto play " + (autoPlay ? "off" : "on" ), true, toggleAutoPlay );
	    if ( isSpeechSupported() ) {
	    	contextMenu.addMenuItem( "Turn speech " + (readAloud ? "off" : "on"), true, toggleSpeech );
	    	contextMenu.addMenuItem( "Speak slower", (readAloud && speechRate > 0.2), speakSlower );
	    	contextMenu.addMenuItem( "Speak faster", (readAloud && speechRate < 4), speakFaster );
	    }
	}
	
	function speakFaster() {
		setSpeechRate( speechRate * 1.1 );
	}
	
	function speakSlower() {
		setSpeechRate( speechRate * 0.9 );
	}
	
	function setSpeechRate( newValue ) {
		cancelSpeaking();		
		speechRate = newValue ;
		dataManager.setLocalItem( 'speechRate', newValue );
	    sayCurrentCard( currentCardIndex, currentSide );
	}


	var autoPlayTimer = null ;
	var autoPlayDelay = 1500 ;
	
    function toggleAutoPlay() {
		autoPlay = !autoPlay ;
        if ( autoPlayTimer != null ) {
             clearTimeout( autoPlayTimer );
             autoPlayTimer = null ;
        }
        doAutoPlay();
        dataManager.setLocalItem( 'fcAutoPlay', autoPlay ? 'Y' : 'N' );
    }

    function autoStep() {
       if ( currentSide + 1 == studyStack.getNumSides() ) {
            moveCurrentCardToPile( remainingCards, getNextCard );
            return 1000;
       } else {
            flipCard();
            return 0;
       }
    }
    
    function doAutoPlay() {
        if ( autoPlayTimer != null ) {
            clearTimeout( autoPlayTimer );
            autoPlayTimer = null ;
        }

    	if ( readAloud ) {
    		return ;
    	}

        var extraDelay = 0;
        if ( autoPlay ) {
        	
            extraDelay = autoStep();
            
            autoPlayTimer = setTimeout( doAutoPlay, autoPlayDelay + extraDelay );
        }
    }

	
	
	
	
	
	function shuffleCards() {
	     var delay = 0;
	     if ( currentCardIndex >= 0 ) {
   	          moveCurrentCardToPile( remainingCards );
   	          delay = 400 * ANIMATE_SPEED;
   	     }
   	     setTimeout( function() {
   	         console.log( 'shuffling remaining------------------' );
   	         var pilesCards = cardIndexes[ 'remaining' ];
   	         shuffle( pilesCards );
   	         initPileTopWithCard( remainingCards, pilesCards[ pilesCards.length - 1] );
   	         shakeRemainingPile();
   	         setTimeout( getNextCard, 400 * ANIMATE_SPEED ); 
   	     }, delay );
	}
	
	function retryIncorrect() {
	     var delay = 0;
	     
	     if ( currentCardIndex >= 0 ) {
   	          moveCurrentCardToPile( remainingCards );
   	          delay = 200 * ANIMATE_SPEED;
   	     }
	     
	     setTimeout( moveIncorrectToRemaining, delay );
	     setTimeout( getNextCard, 500+delay ); 
	}
	
	
	function reverseSides() {
	     reverseOrder = !reverseOrder ;
	     currentSide = 0;
	     refreshUI();
		 saveBoxStates();	     
	}
	

	function moveCard( card, destination, thenDoFunc ) {
	     var ccIndex = currentCardIndex ;
	     if ( card == currentCard ) {
	     	currentCardIndex = -1;
	     }
	     var movingCard = $( '#fc_movingCard' );
         var destId = destination.attr( 'id' );
         movingCard.css( 'zIndex', destId=='remainingCards' ? 25 : 75 );
         
	     movingCard.css( card.offset() );    // ??? what does this do???
	     
	     movingCard.css( 'background-color', card.css( 'background-color' ) );     
	     movingCard.css( 'font-size', card.css( 'font-size' ) );
	     movingCard.width( card.width() );
	     movingCard.height( card.height() );
	
	     movingCard.html( card.html() );
	     movingCard.show();
	     if ( card == currentCard ) {
	     	card.css({ opacity: 0.0 });   // hide card
	     }
	
	     var properties = destination.offset();
	     properties.height = destination.height();
	     properties.width = destination.width();
	     properties.fontSize = destination.css('font-size');
	     properties.scale = 0.2; 
	     movingCard.animate( properties, 200 * ANIMATE_SPEED, 'linear', function() {
	         if ( destination == currentCard ) {
	         	  initCurrentCardHtml( ccIndex, currentSide );
	         }
	         else {
    	         destination.css( 'backgroundColor', movingCard.css( 'backgroundColor' ) );
	             destination.html( movingCard.html() );
	         }
	         movingCard.hide();
	         destination.css({ opacity: 1.0 });   // show destination
	         if ( destId != "fc_currentCard" ) {
	         	var pileName = destId.substr( 0, destId.length - "cards".length );
				var pilesCards = cardIndexes[ pileName ];
	         	if ( pileName == 'remaining' ) {
					// put current card at back of remaining cards
                    pilesCards.splice(0,0,ccIndex);
				}
				else {
					// put current card at front of pile
					pilesCards[ pilesCards.length ] = ccIndex ;
				}
	         	
	         	adjustCount( pileName, 1 );
	         	if ( thenDoFunc ) {
	         	     thenDoFunc();
	         	}
	         	
	         }
	         
	
	     } );
	}


           function restartAll() {
        	    clearState();
                startOver();
           }
           
           function clearState() {
               dataManager.setLocalItem( getStateVarName( studyStack.id ), '' );
               clearPassComplete();
           }


	   function startOver() {
	        console.log( 'startOver' );
	        retries=0;
	
	        currentSide = 0;
	        
	        elapsedTime = 0;
	        startTime = new Date().getTime()  ;
	        moveHistory = new Array();
	        var delay = 0;
	        var curOpacity = currentCard.css( 'opacity' );
	        console.log( 'curOpacity is ' + curOpacity + '<<<<<' );
	        if ( curOpacity > 0.9 ) {
	             console.log( 'startOver clearing currentCard to remainingCards' );
         	     moveCurrentCardToPile( remainingCards );
	             delay++;
	        }
	        if ( getCount( 'correct' ) > 0 ) {
	            console.log( 'startOver calling moveCorrectToRemaining' );
	            setTimeout( moveCorrectToRemaining, delay * 100 * ANIMATE_SPEED );
	            delay++;
	        }
	        if ( getCount( 'incorrect' ) > 0 ) {
	            console.log( 'startOver calling moveIncorrectToRemaining' );
	            setTimeout( moveIncorrectToRemaining, delay*100*ANIMATE_SPEED );
	            delay ++;
	        }
	        setTimeout( finishStart, delay*300*ANIMATE_SPEED );
	        //debug( 'startOver complete' );
	        
	        setTimeout( doAutoPlay, 500 );

	     }



         function finishStart() {
		       cardIndexes = new Array();
		       cardIndexes[ 'correct' ] = createInitialOrderArray( 0 ) ;
		       cardIndexes[ 'incorrect' ] = createInitialOrderArray( 0 ) ;
		       cardIndexes[ 'remaining' ] = createInitialOrderArray( studyStack.getNumCards() ) ;
		
		       setCount( 'remaining', studyStack.getNumCards(), true );
		       setCount( 'correct', 0, true);
		       setCount( 'incorrect', 0, true);
		
               var savedSessionData = dataManager.getLocalItem( getStateVarName( studyStack.id ) );
               if (  savedSessionData != null && savedSessionData.length > 10 ) {
		          console.log( 'savedSessionData.length=' + savedSessionData.length ) ;
		          restoreBoxStates( savedSessionData );
		       }
		       else {
		           shakeRemainingPile();
		           var pilesCards = cardIndexes[ 'remaining' ];
		           var pile = remainingCards ;
		           initPileTopWithCard( pile, pilesCards[ pilesCards.length - 1] );
		           console.log( 'finishStart calling getNextCard' );
		           setTimeout( getNextCard, 700*ANIMATE_SPEED );
		       }
		 }
		 
		 function refreshUI() {
             for ( var p=0; p<pileNames.length; p++ ) {
                   var pileName = pileNames[p]; 		 
		           var pilesCards = cardIndexes[ pileName ];
		           if ( pilesCards.length > 0 ) {
                   	   var pile = $( '#' + pileName + 'Cards' );
     		           initPileTopWithCard( pile, pilesCards[ pilesCards.length - 1] );
     		       }
             }
             
             if ( currentCardIndex >= 0 ) {
             	initCurrentCardHtml( currentCardIndex, currentSide );
             }
		 }
		 

  	     function createInitialOrderArray( size ) {
	        var orderArray = new Array();
	        var i;
	        for ( i=0; i<size; i++ ) {
	           orderArray[i] = size-i-1;
	        }
	        return orderArray ;
	     }
		 
		 
    	 var cardsProcessed = 0;
		 function getNextCard() {
			 
			 speechId = '' ;
			 
		     if ( (getCount( 'remaining' ) == 0) && currentCardIndex < 0 ) {
		          onPassComplete() ;
		     }
		     else {
		          moveToCenter( 'remaining' );
		          cardsProcessed++;
		     }
		 }


   	     function moveToCenter( fromPileName ) {
   	         clearPassComplete() ;
   	         if ( currentCardIndex >= 0 ) {
   	              moveCurrentCardToPile( remainingCards, function() {
   	                  reallyMoveToCenter( fromPileName );
   	              } );
   	         }
   	         else {
   	             reallyMoveToCenter( fromPileName );
   	         }
		 }
		 
		 function reallyMoveToCenter( fromPileName ) {
   	         console.log( 'reallyMoveToCenter fromPileName=' + fromPileName );
   	         
		     var count = getCount( fromPileName ) ;
		     if ( count > 0 ) {
		 		 currentCard.removeAttr('style');
		
		         currentSide = 0;
		         var pilesCards = cardIndexes[ fromPileName ];
		         currentCardIndex = pilesCards[ pilesCards.length - 1] ;
		         console.log( 'moving ' + currentCardIndex + ' to center ' );
		         pilesCards.length--;
		
                 var pile = $( '#' + fromPileName + 'Cards' );
                 console.log( 'moveToCenter fromPileName=' + fromPileName + ' pile=' + pile + ' currentCard=' + currentCard ); 
		         moveCard( pile, currentCard );
		         adjustCount( fromPileName, -1 );
		         if ( count > 1 ) {
		              initPileTopWithCard( pile, pilesCards[ pilesCards.length - 1] );
		         }
		      }
		 }
		 
		 
	function shakeRemainingPile() {
	    console.log( 'shakeRemainingPile*********************************' );
   		remainingCards.animate({ left: 5, top: -3 }, 50)
          .animate({ left: 0, top: 0 }, 50)
          .animate({ left: -5, top: 3 }, 50)
          .animate({ left: 0, top: 0 }, 50)
          .animate({ left: 5, top: -3 }, 50)
          .animate({ left: 0, top: 0 }, 50);
    }
	 

   function animateMovingCardsToRemaining( srcPile, duration, completeFunc ) {
        var destProps = getAnimateToRemainingProps( srcPile );
        srcPile.animate( destProps, duration, 'linear', completeFunc );
   } 


   function moveIncorrectToRemaining() {
       var numIncorrect = getCount( 'incorrect' );
       if ( numIncorrect > 0 ) {
          incorrectCards.css( 'zIndex', 10 );
		  var prevIncorrectCards = cardIndexes[ 'incorrect' ];
		  cardIndexes[ 'incorrect' ] = new Array();
          setCount( 'incorrect', 0, false );
          animateMovingCardsToRemaining( incorrectCards, 200*ANIMATE_SPEED, function() {
              finishMoveIncorrectToRemaining( prevIncorrectCards );
          } );
       }
   }
   
   
   
   function finishMoveIncorrectToRemaining( prevIncorrectCards ) {
       incorrectCards.css({ opacity: 0.0 });   // hide card

       shuffle( prevIncorrectCards );

       var pilesCards = cardIndexes[ 'remaining' ];
       for ( var i=0; i<prevIncorrectCards.length; i++ ) {
            pilesCards[ pilesCards.length ] = prevIncorrectCards[i] ;
       }
       
       adjustCount( 'remaining', prevIncorrectCards.length );
       incorrectCards.animate({ left: 0 }, 0);

       var pile = $( '#remainingCards' );
       initPileTopWithCard( pile, pilesCards[ pilesCards.length - 1] );
       incorrectCards.css( 'zIndex', 3 );
   }




   var numCorrectCardsMoved ;
   function moveCorrectToRemaining() {
       numCorrectCardsMoved = getCount( 'correct' );
       console.log( 'moveCorrectToRemaining count=' + numCorrectCardsMoved );
       if ( numCorrectCardsMoved > 0 ) {
          setCount( 'correct', 0, false );
          animateMovingCardsToRemaining( correctCards, 200*ANIMATE_SPEED);
          setTimeout( finishMoveCorrectToRemaining, 200*ANIMATE_SPEED );
       }
   }

   function finishMoveCorrectToRemaining() {
       //adjustCount( 'remaining', numCorrectCardsMoved );
       numCorrectCardsMoved = 0;
       correctCards.css({ opacity: 0.0 });   // hide card
       correctCards.animate({ left: 0 }, 0);
   }


    function getPositionProps( pile ) {
        var properties=new Object();
        properties.top=pile.css('top');
        properties.left=pile.css('left');
        properties.width=pile.css('width');
        properties.height=pile.css('height');
        return properties;
     }
     


	function getAnimateToRemainingProps( pile ) {
          var properties=new Object();
          properties.left = remainingCards.offset().left - pile.offset().left ;  
          return properties;
	}


   function restoreBox( boxName, savedIndexes ) {
       //debug( 'restoring ' + boxName + ' with ' + savedIndexes );
       cardIndexes[ boxName ] = savedIndexes ;
       setCount( boxName, cardIndexes[boxName].length, true );
       //showPileTop( boxName );
    }


    function indexToId( index ) {
    	if ( studyStack != null ) {
    		var row = studyStack.getRow( index ) ;
    		if ( row ) {
    			return row.cardId ;
    		}
    	}
		
        return -1 ;
    }
    
    function idToIndex( id ) {
		if ( id == '' ) {
			return -1;
		}
        var numCards = studyStack.getNumCards();
        for ( var index = 0; index<numCards; index++ ) {
            if ( studyStack.rows[index].cardId == id ) {
                return index ;
            }
        }
        return -1;
    }
    
    function indexArrayToIdArray( indexArray ) {
        var idArray = [];
        for ( var i=0; i<indexArray.length; i++ ) {
            idArray[ idArray.length ] = indexToId( indexArray[i] ); 
        }
        return idArray ;
    }
    
    function idArrayToIndexArray( idArray ) {
        var indexArray = [];
        for ( var i=0; i<idArray.length; i++ ) {
            var id = idArray[i];
			if ( id != '' ) {
            	var index = idToIndex( id );
            	if ( index>= 0 ) {
	                indexArray[ indexArray.length ] = index ;
	            }
				else {
					console.log( 'id ' + id + ' not found' );
				}
			}
        }
        return indexArray ;
    }
    function getStateVarName( studyStackId ) {
        return 'fcstate' + studyStackId 
   }


   function inArray( theArray, theValue ) {
        for ( var i=0; i<theArray.length; i++ ) {
             if ( theArray[i] == theValue ) {
                 return true ;
             }
        }
        return false ;
   }
   
    
    function saveBoxStates() {
        var saveState = stringifyNumArray( indexArrayToIdArray( cardIndexes[ 'correct' ] ) ) + '|' +
                    stringifyNumArray( indexArrayToIdArray(cardIndexes[ 'incorrect' ] ) ) + '|' +
                    stringifyNumArray( indexArrayToIdArray(cardIndexes[ 'remaining' ] ) ) + '|' +
                    stringifyNumArray( indexArrayToIdArray(moveHistory) ) + '|' +
                    (reverseOrder ? 1 : 0) + '|' +
                    indexToId( currentCardIndex ) + '|' +
                    retries + '|' +
                    elapsedTime;
        dataManager.setLocalItem( getStateVarName( studyStack.id ), saveState );
        console.log( 'saveBoxState as ' + saveState );
    }

 
    function restoreBoxStates( saveState ) {
         //debug( 'restoring ' + saveState );
         var savedParts = saveState.split( '|' );
         var correctCards =   idArrayToIndexArray( savedParts[0].split( "," ) );
         var incorrectCards = idArrayToIndexArray( savedParts[1].split( "," ) );
         var remainingCards = idArrayToIndexArray( savedParts[2].split( "," ) );

         currentCardIndex = idToIndex( savedParts[5] );

         for ( var index = 0; index< studyStack.getNumCards(); index++ ) {
              if ( index != currentCardIndex &&
                   !inArray( correctCards, index ) &&
                   !inArray( incorrectCards, index ) &&
                   !inArray( remainingCards, index ) ) {
                remainingCards[ remainingCards.length ] = index ;
              }
         }

         reverseOrder = (1==savedParts[4]);
         console.log( 'restored reverseOrder as ' + reverseOrder );

         restoreBox( 'correct', correctCards );
         restoreBox( 'incorrect', incorrectCards );
         restoreBox( 'remaining', remainingCards );

         moveHistory = savedParts[3].split( "," );
         //debug( 'moveHistory length is ' + moveHistory.length );
         //debug( 'currentSide=' + currentSide );
         retries = savedParts[6];
         elapsedTime = savedParts[7];
    //   displayElapsedTime();
         //debug( 'currentCardIndex = ' + currentCardIndex );
         if ( currentCardIndex == -1 ) {
                getNextCard();
         }
         else {
             initCurrentCardHtml( currentCardIndex, currentSide );
         }
    }

    function prepareFlipper() {
    	if ( fcCardFlipper == null ) {
    		curSides = [];
	        currentCard.html( 
	        	'<div class="flip" >' + 
			        '<div id="fcCardFlipper" class="fc_card" >' + 
			            '<div id="fcFront" class="face"></div>' + 
			            '<div id="fcBack" class="face"></div>' + 
			        '</div>' + 
			    '</div>' 
			); 
	        fcCardFlipper = document.getElementById( 'fcCardFlipper' );
	        curSides[0] = $( '#fcFront' );
	   	    curSides[1] = $( '#fcBack' );
        }
 	   degrees = 0;
       fcCardFlipper.style.webkitTransform='rotateX(' + degrees + 'deg)';
    	
    }

    var showingPassComplete = false ;
    
    function clearPassComplete() {
        if ( showingPassComplete ) {
			 passCompleteDiv.css( 'display', 'none');
		     currentCard.css({ opacity: 0.0 });  
             currentCard.css( 'display', 'block' );
        }
        showingPassComplete = false ;
    }
    
    function onPassComplete() {
       showingPassComplete = true ;
       // show percent done
       var numIncorrect = getCount( 'incorrect' );
       var numCorrect = getCount( 'correct' );
       var min = Math.min( currentCard.height(), currentCard.width() );
       var radius = (min - 20) / 2;
       
       passCompleteDiv.html( "<div id='fcStats'>" + svgDonut( radius, radius/2, 10, 1.0*numCorrect / ( numCorrect + numIncorrect), "green","#ee0022", "#ffffff" ) + "</div>");
       
       currentCard.css( 'display', 'none' );
       passCompleteDiv.css( 'display', 'block');
       
        if ( numIncorrect == 0 ) {
		     studyStack.finishActivity( FLASHCARD_CODE );
		}
	}

	function onOrientationChange() {
		if ( currentCardIndex == -1 ) {
			onPassComplete();
		}
	}


	
      return {
          onOrientationChange: onOrientationChange,
          startPage: startPage,
          endPage: endPage,
          saveBoxStates: saveBoxStates,
          restartAll: restartAll,
          clearState: clearState,
      } ;

})();

	
