console.log( 'hungrybug.js' );

var hungryBugPage = (function() {

	function startPage() {
       chaseContainer = document.getElementById( "chaseContainer" );
       hungryBugPageDiv = document.getElementById( 'hungryBugPage' );
	   calculateSize();
		
	   if ( studyStack.getNumCards() < 4 || studyStack.getNumSides() < 2 ) {
	        chaseContainer.innerHTML = "<div id='activityError'>The Hungry Bug activity requires at least four cards with at least two sides.</div>" ;
            document.getElementById( 'gameOver' ).style.display = 'none';
	   }
	   else {
           contextMenu.setSetupFunc( setupContextMenu );
           dPad.show( processGameKey );
           startGame();
	   }
	}
	
	function endPage() {
	     pauseGame();
	     hideProblem();
	     dPad.hide();
	}
	

	function setupContextMenu() {
	    contextMenu.addMenuItem( "Start Over", true, startGame );
	}
	
        var BUG_SIZE=22;  // in pixels
        var playing = false ;
        var playButton;
        var chaseContainer;
        var questionWrapper = document.getElementById( "hungryBugQuestionWrapper" );
        var eatButton = document.getElementById( 'eat' );
        var scoreDiv;
        var bugSprites = new Array() ;
        var answerSprites ;
        var timerId ;
        var me ;
        var NO_REBOUND = 0;
        var RANDOM_REBOUND = 1;
        var FLY_THRU_EDGES = 2;
        var userVelocity = BUG_SIZE;
        var gameLevel ;
        var score ;
        var timerDelay ;
        var problem ;
        var NUM_WRONG_ANSWERS = 2;
        var SNAKE_LENGTH_FOR_PIE = 10;
        var NUM_ANSWERS = 3;
        var bodyParts ;   // front of array is tail, end of array is neck
        var alive ;
        var questionDisplayed = false ;
        var hungryBugPageDiv ;
        
        document.getElementById( 'gameOver' ).onclick = startGame ;

        function createMeImg() {
             var img = document.createElement( 'img' );
             img.src = 'images/head.png';
             img.alt = '';
             img.id = 'me';
             img.className = 'sprite';
             img.style.zIndex = 10;
             img.style.height = BUG_SIZE + 'px';
             return img ;
        }       
        
        function startGame() {
            document.getElementById( 'gameOver' ).style.display = 'none';
            priorMeX = -1 ;
            priorMeY = -1 ;
            alive = true ;
            bodyParts = new Array();
            timerDelay = 500 ;
            scoreDiv = document.getElementById( "score" );
            eatButton.onclick = hideProblem ;
			chaseContainer.innerHTML = '' ;
			
            gameLevel = 1;
            score = 0;
 
            bugSprites = new Array();
 
            var meImg = createMeImg();
            me = new Sprite( meImg, new Point( 220, 190 ) );
            chaseContainer.appendChild( meImg );
            
            answerSprites = new Array();
            answerSprites[0] = newAnt( "food1.png" );
            answerSprites[1] = newAnt( "food2.png" );
            answerSprites[2] = newAnt( "food3.png" );
                        
            bugSprites = new Array();
            
            startNextProblem();

            growBody();
            playGame();
        }

        var MIN_SQUARES = 200 ;
        var MIN_BUG_SIZE = 20 ;
        var MAX_BUG_SIZE = 48 ;
        
        function calculateSize() {
            var area = hungryBugPageDiv.clientHeight * hungryBugPageDiv.clientWidth ;

            // want BUG_SIZE to be no more than MAX_BUG_SIZE pixels
            BUG_SIZE = MAX_BUG_SIZE ;
            
            // want at least MIN_SQUARES squares   website has 391 squares
            while ( (area / (BUG_SIZE * BUG_SIZE) < MIN_SQUARES) && BUG_SIZE>MIN_BUG_SIZE ) {
                BUG_SIZE -- ;
            }    
            console.log( "BUG_SIZE is " + BUG_SIZE + " num squares = " + area/(BUG_SIZE*BUG_SIZE) );
            
            chaseContainer.style.width = Math.floor( chaseContainer.clientWidth / BUG_SIZE ) * BUG_SIZE ;
            chaseContainer.style.height = Math.floor( chaseContainer.clientHeight / BUG_SIZE ) * BUG_SIZE ;
        }
        
        function changeFood() {
            var foods = [ 'banana', 'pizza', 'hamburger' ];
            shuffle( foods );
            for ( var i=0; i<3; i++ ) {
                var foodFile = "images/" + foods[i] + '.png' ;
                answerSprites[i].object.src =  foodFile ;
                document.getElementById( "food" + (i+1)  ).src = foodFile ;
            }
        }

        
        function startNextProblem() {
          if ( alive ) {
              changeFood();
              problem = getNextProblem();
              assignBugs( problem );
              showProblem( problem );

              var i; 
              for ( i = 0; i<answerSprites.length; i++ ) {
                   var answerSprite = answerSprites[i];
                   if ( answerSprite.object.parentNode != chaseContainer ) {
                       chaseContainer.appendChild( answerSprite.object );
                       bugSprites[bugSprites.length] = answerSprite ;
                   }
              }
              if ( (numCorrect > 0) && ((numCorrect % 5) == 0) ) {
                  timerDelay = timerDelay * 0.9;
              }              
           }
        }
        


       function assignBugs( problem ) {
            shuffleQuestionOrder();
            for ( var i=0; i<NUM_ANSWERS; i++ ) {
                randomlyMoveSprite( answerSprites[i] );
                var q = questionOrder[i];
                if ( q == NUM_WRONG_ANSWERS ) {
                    // correct answer is at order i
                    problem.correctAnswer.order = i;
                    problem.correctAnswer.bug = answerSprites[i] ;
                }
                else {
                    if ( q < problem.incorrectAnswers.length ) {
                         // incorrect answer q is at order i
                         problem.incorrectAnswers[q].order = i;
                         problem.incorrectAnswers[q].bug = answerSprites[i];
                    }
                    else {
                         removeSprite( answerSprites[i] );
                    }
                }
            }
        }


        function hitFood( food ) {
              return ( me.object.style.top == food.object.style.top ) &&
                     ( me.object.style.left == food.object.style.left );
        }

        function hitBadFood() {
            var i;
            for ( i=0; i<problem.incorrectAnswers.length; i++ ) {
               if ( hitFood( problem.incorrectAnswers[i].bug ) ) {
                    return i ; 
               }
            }
            return -1 ;
        }
        function hitCorrectFood() {
              return hitFood( problem.correctAnswer.bug );
        }
    



        var questionOrder = [ 0, 1, 2 ];
       
        function shuffle( array ) {
            var i=0;
            for ( i=0; i<array.length; i++ ) {
                var r = Math.floor(Math.random() * NUM_ANSWERS) % NUM_ANSWERS;
                var tmp = array[r];
                array[r] = array[i];
                array[i] = tmp;
            }
        } 


        function shuffleQuestionOrder() {
            shuffle( questionOrder );
        }
                
        
        function showProblem( problem ) {
              eatButton.disabled = true ;
              document.getElementById( 'bugQuestion' ).innerHTML = escapeHtml(problem.question);
              showPossibleAnswer( problem.correctAnswer.order, problem.correctAnswer.text );
              for ( var a = 0; a<problem.incorrectAnswers.length; a++ ) {
                   var incorrectAnswer = problem.incorrectAnswers[a] ;
                   showPossibleAnswer( incorrectAnswer.order, incorrectAnswer.text );
              }
              a++ ;
              while ( a<NUM_WRONG_ANSWERS ) {
                  var answerObj = document.getElementById( 'answer' + a );
                  answerObj.parentNode.style.display = 'none' ;
                  a++;   
              }
              displayProblem();
              setTimeout( enableEatButton, 500 );
        }
        
        function enableEatButton() {
        	eatButton.disabled = false ;
        }
        
        function displayProblem() {
            questionDisplayed = true ;
            questionWrapper.style.display = 'table' ;
        }
        
        function hideProblem() {
        	questionWrapper.style.display = 'none';
        	questionDisplayed = false ;
        }
        
        function showPossibleAnswer( i, text ) {
              var answerObj = document.getElementById( 'answer' + i );
              answerObj.className = 'answer';
              answerObj.innerHTML = escapeHtml(text);
              answerObj.style.textDecoration = '';
              answerObj.parentNode.style.display = '' ;
              answerObj.parentNode.style.backgroundColor = '';
        }
              
        
        function Sprite( object, centerPoint ) {
            this.object = object;
            this.xIncrement = 0.0;
            this.yIncrement = 0.0;

            moveSprite( this, centerPoint.y, centerPoint.x );
        }

        
        
        function newAnt( fileName ) {
            var img = document.createElement( 'img' );
            img.className = "sprite" ;
            img.src = "images/" + fileName ;
            img.alt = fileName ;
            img.style.height = BUG_SIZE + 'px';
            img.onclick = displayProblem ;
            var ant = new Sprite( img, new Point( 0, 0 ) ); 
            randomlyMoveSprite( ant );
            return ant ;
        }


	function isCloseToMe( top, left ) {
            return  Math.abs( top - me.center.y ) < 3 || Math.abs( left - me.center.x ) < 3  ;
        }

        function onTopOfAnyBug( top, left ) {  
              for ( var i = 0; i<answerSprites.length; i++ ) {
                   if ( (answerSprites[i].center.x == left) &&
                        (answerSprites[i].center.y == top ) ) {
                        return true ;
                   } 
              }
              return false ;
        }

        function randomlyMoveSprite( sprite ) {
            do {
               var top = Math.random() * (chaseContainer.clientHeight - BUG_SIZE) ;
               var left = Math.random() * (chaseContainer.clientWidth - BUG_SIZE);
               top = Math.round((top / BUG_SIZE) * BUG_SIZE) ;
               left = Math.round((left / BUG_SIZE) * BUG_SIZE) ;
            } while ( isCloseToMe( top, left ) || onTopOfAnyBug( top, left )  ) ;
            moveSprite( sprite, top, left );
        }

        function moveSprite( sprite, top, left ) {
            sprite.center = new Point( left, top );
            sprite.center.x = Math.round(sprite.center.x / BUG_SIZE ) * BUG_SIZE ;
            sprite.center.y = Math.round(sprite.center.y / BUG_SIZE ) * BUG_SIZE ;
            sprite.object.style.left =  sprite.center.x  + "px";
            sprite.object.style.top = sprite.center.y  + "px";
        }
        
        
        function Point( x, y ) {
            this.x = x ;
            this.y = y ;
        }

        function Line( point1, point2 ) {
            this.point1 = point1 ;
            this.point2 = point2 ;
        }
        
        function togglePlay() {
           if ( playing ) {
               pauseGame();
           }
           else {
               playGame();
           }
        }

        
        function pauseGame() {
            playing = false ;
            console.log( 'pauseGame' );
            clearTimeout( timerId );
            timerId = null;
        }
        
        function playGame() {
            playing = true ;
            moveObjects( true );
        }
        
       function endGame() {
            console.log( 'endGame' );
            markCorrectAnswer()
            me.object.style.display = 'none';
            removeAllSprites();
            removeSprite( me );
            clearTimeout( timerId );
            timerId = null;
            timerDelay = 0;
            setTimeout( gameOver, 4000 );
            document.getElementById( 'gameOver' ).style.display = 'block';
       }
       
       function gameOver() {
           setTimeout( function() { onGameOver( score ); }, 500  );
       }
        


        function isGameKey( key ) {
            return playing && ( key == LEFT_ARROW_KEY  || key == RIGHT_ARROW_KEY || 
                 key == UP_ARROW_KEY || key == DOWN_ARROW_KEY  ||
                 key == J_KEY || key == K_KEY ||
                 key == L_KEY || key == H_KEY );
        }        
        
        function debug( msg ) {
        }
      
        function gameClickPos( event ) {
            
        } 


        function goLeft() {
           me.xIncrement = -BUG_SIZE ;
           me.yIncrement = 0;
        }
        function goRight() {
            me.xIncrement = BUG_SIZE ;
            me.yIncrement = 0;
        }
        function goUp() {
            me.xIncrement = 0;
            me.yIncrement = -BUG_SIZE;
        }
        function goDown() {
            me.xIncrement = 0;
            me.yIncrement = BUG_SIZE;
        }
  
        var priorMeX ;
        var priorMeyY ; 
        function hasMovedSinceLastKey() {
           var hasMoved = me.center.x != priorMeX || me.center.y != priorMeY ;
           priorMeX = me.center.x ;
           priorMeY = me.center.y ;
           return hasMoved ;
        }

        function processGameKey( key ) {
            if ( hasMovedSinceLastKey() ) {
	             if ( me.xIncrement == 0 ) {
	                 if ( key == LEFT_ARROW_KEY || key == H_KEY ) {
						goLeft();
	                 } else if ( key == RIGHT_ARROW_KEY || key == L_KEY ) {
						goRight();
	                 }
	             }
	             if (me.yIncrement == 0 ) {
	                 if ( key == UP_ARROW_KEY || key == K_KEY ) {
						goUp();
	                 } else if ( key == DOWN_ARROW_KEY || key == J_KEY ) {
						goDown();
	                 }
	             }
            }
        }
        
      
        
        
        function murderEvent(evt) { 
            evt.cancel=true; 
            evt.returnValue=false; 
            evt.cancelBubble=true; 
            if (evt.stopPropagation) evt.stopPropagation(); 
            if (evt.preventDefault) evt.preventDefault(); 
            return false;
        }        
        
        
        function hitSelf() {
            var meTop = me.object.style.top ;
            var meLeft = me.object.style.left ; 
            var i;
            for ( i=bodyParts.length-2; i>0; i-- ) {
                var bodyPart = bodyParts[i] ;
                if ( bodyPart.style.top == meTop && bodyPart.style.left == meLeft ) {
                      console.log( 'hitSelf returning true' );
                      return true ;
                }
            }
            return false ;
        } 
                    
        function hitEdge( sprite, containerDiv ) {
             var hitEdge = false ;
             var maxX = containerDiv.clientWidth - sprite.object.clientWidth  ;
             var maxY = containerDiv.clientHeight - sprite.object.clientHeight ;
             if ( maxX<=0 || maxY<=0 ) {
                  console.log( '   maxX=' + maxX + ' containerDiv.clientWidth=' + containerDiv.clientWidth + ' sprite width=' + sprite.object.clientWidth + ' center x=' + sprite.center.x);
                  console.log( '   style.display=' + containerDiv.style.display );
                  console.trace();
             }
             
             if ( sprite.center.x < 0 ) {
                sprite.center.x = 0 ;
                hitEdge = true ;
             }
             if ( sprite.center.y < 0 ) {
                sprite.center.y = 0 ;
                hitEdge = true ;
             }
             if ( sprite.center.x > maxX ) {
                sprite.center.x = maxX ;
                hitEdge = true ;
             }
             if ( sprite.center.y > maxY ) {
                sprite.center.y = maxY ;
                hitEdge = true ;
             }
             if ( hitEdge ) { console.log( 'hitEdge returning ' + hitEdge ); }
             return hitEdge ;
        }
        

        function growBody() {
            var newTail = document.createElement( 'img' );
            newTail.className = 'sprite';
            newTail.src = 'images/gcircle.png';
            newTail.style.left = me.object.style.left ;
            newTail.style.top = me.object.style.top ;
            newTail.style.height = BUG_SIZE + 'px';
            
            var startArray = new Array();
            startArray[0] = newTail ;
            bodyParts = startArray.concat( bodyParts );
            chaseContainer.appendChild( newTail );
            if ( bodyParts.length > SNAKE_LENGTH_FOR_PIE )  {
            	studyStack.finishActivity( HUNGRYBUG_CODE );
            }
        }

        function shrinkBody() {
            var tail = bodyParts.shift();
            tail.parentNode.removeChild( tail );
        }
                
        function moveHead( sprite ) {
           console.log( 'sprite xinc=' + sprite.xIncrement );
           console.log( 'sprite yinc=' + sprite.yIncrement );
           sprite.center.x = sprite.center.x + sprite.xIncrement ;
           sprite.center.y = sprite.center.y + sprite.yIncrement ;
           var died = hitEdge( sprite, chaseContainer )
           sprite.object.style.left = Math.round( sprite.center.x ) + "px";
           sprite.object.style.top = Math.round( sprite.center.y ) + "px";
           died = died || hitSelf();
           if ( died ) {
                sprite.object.style.display  = 'none' ;
                onDeath() ;
            }
            else {
                if ( hitCorrectFood() ) {
                    onCorrectAnswer();
                }
                else {
                    var badFoodNum = hitBadFood();
                    if ( badFoodNum >= 0 ) {
                        onIncorrectAnswer( badFoodNum );
                    }
                }
            }
        }
  
        function onDeath() {
             console.log( 'onDeath' );
             alive = false ;
             shrinkContinuously(); 
             endGame();
        }

        function shrinkContinuously() {
            if ( bodyParts.length > 0 ) {
                 shrinkBody();
                 setTimeout( shrinkContinuously, 50 );
            }
        }


        function moveBody() {
            var tail = bodyParts.shift();
            if ( tail != null ) {	
                var newNeck = tail ;
                newNeck.style.left = me.object.style.left ;
                newNeck.style.top = me.object.style.top ;
                bodyParts[ bodyParts.length ] = newNeck ;
           }
        }

                
        function moveObjects( repeat ) {
            clearTimeout( timerId );
            
            console.log( 'moveObjects repeat=' + repeat + ' alive=' + alive + ' timerDelay=' + timerDelay + ' playing=' + playing );
            if ( !playing ) {
                return ;
            }
            
            console.trace();
            if ( alive ) { 
                if ( repeat ) {
                    timerId = setTimeout( function() { 
//                            console.log( 'in timeout function of moveObjects' );
                            moveObjects( true ); 
                        },  
                        timerDelay 
                    );
                }
                if ( !questionDisplayed ) {
                	moveBody();
                	moveHead( me );
            	}
             }
        }
        
        
        function removeAllSprites() {
           var numSprites = bugSprites.length ;
           for ( var i=0; i<numSprites; i++ ) {
              if ( bugSprites.length > 0 ) {
                  removeSprite( bugSprites[0] );
              }
           }
        } 
        
        function removeSprite( sprite ) {
            // remove the image from the screen
            var pNode = sprite.object.parentNode ;
            if ( pNode ) {
                  pNode.removeChild( sprite.object );
            }
            
            // remove the sprite from our array
            var i;
            for ( i=0; i<bugSprites.length; i++ ) {
                if ( bugSprites[i] == sprite ) {
                     bugSprites.splice( i, 1 );
                     return ;
                }
            }
        }

       var numCorrect = 0;
       var hits = 0;
       
       function onIncorrectAnswer( ansNum ) {
    	     playWrong();
             if ( bodyParts.length > 1 ) {
                shrinkBody();
             }
             markIncorrectAnswer( problem.incorrectAnswers[ansNum].order );
             removeSprite( problem.incorrectAnswers[ansNum ].bug  );
       }


       function onCorrectAnswer() {

             numCorrect++ ;
             playCorrect();
             growBody();
             
             removeSprite( problem.correctAnswer.bug );
             incrementScore(  10 * bodyParts.length ) ;
             setTimeout( hideIncorrectAnswers, 500 );
             markCorrectAnswer();
             
             setTimeout( startNextProblem,  2000 );
 
             if ( onAteBug ) {
                onAteBug();
             } 
       }
       
       function markCorrectAnswer() {
           var answerDiv = document.getElementById( 'answer' + problem.correctAnswer.order ) ;
           answerDiv.className = answerDiv.className + ' correctAnswer' ; 
       }
       
       function markIncorrectAnswer( ansNum ) {
           var answer = document.getElementById( 'answer' + ansNum );
           answer.style.textDecoration = 'line-through';
           answer.parentNode.style.backgroundColor = '#bbbbbb';
       }

       function incrementScore( value ) {
           score += value ;
           if ( score < 0 ) {
               score = 0;
           }
           //scoreDiv.innerHTML = score ;
       }
       
       function hideIncorrectAnswers() {
           var i;
           for ( i=0; i<problem.incorrectAnswers.length; i++ ) {
               var ansNum = problem.incorrectAnswers[i].order ;
               var answer = document.getElementById( 'answer' + ansNum );
               answer.parentNode.style.display = 'none' ;
           }
       }


       

       function findBadBug( bug ) {
            var i=0;
            for ( i=0; i<problem.incorrectAnswers.length; i++ ) {
                if ( problem.incorrectAnswers[i].bug == bug ) {
                     return i;
                }
            }
            return -1;
       }       






       var bugsEaten = 0;
       function onAteBug() {
          bugsEaten ++;
       }

       var lastProblem = -1;

       function getNextProblem() {
            var p;
            var i=0;
            do {
               p = Math.floor( Math.random() * studyStack.rows.length );
            } while ( (lastProblem == p) && (i++ < 5));
            lastProblem = p;

            var ansCol = randomInteger( 1, studyStack.fields.length );
            var questionText = studyStack.getCardSide( p, 0 );
            var correctAnswer = studyStack.getCardSide( p, ansCol );
            var problem = { questionHeading: studyStack.fields[0].name + ":",
                            answerHeading: studyStack.fields[ansCol].name + ":",
                            question: questionText,
                            correctAnswer: { text: correctAnswer},
                            incorrectAnswers: getIncorrectAnswers( correctAnswer, ansCol )  };
            return problem ;
       }

       function problemToString( problem ) {
            return '{ questionHeading:' + problem.questionHeading +
                     ', answerHeading:' + problem.answerHeading +
                     ', question: ' + problem.question +
                     ', correctAnswer: ' + problem.correctAnswer.text +
                     '}';

       }

     function getIncorrectAnswers( correctAnswer, column ) {
             var wrongAnswers = new Array();
             for ( var numTries =0; numTries < 20; numTries++ ) {
                  var p = Math.floor( Math.random() * studyStack.rows.length );
                  var wrongAnswer = studyStack.getCardSide( p, column );
                  if ( wrongAnswer != correctAnswer ) {
                      // make sure it's not one of the wrong answers we already have
                      var n ;
                      for ( n=0; n<wrongAnswers.length; n++ ) {
                           if ( wrongAnswer == wrongAnswers[n].text ) {
                               break;
                           }
                      }
                      if ( n == wrongAnswers.length ) {
                          // we made it to the end w/o breaking, so this answer hasn't been used yet
                          wrongAnswers[ wrongAnswers.length ] = {text: studyStack.getCardSide(p,column)};
                      }
                  }
                  if ( wrongAnswers.length == 2 ) {
                      break;
                  }
             }
             return wrongAnswers;
       }

       function onGameOver( score ) {
       }

       
       
      return {
          endPage: endPage,
          startPage: startPage,
          pausePage: pauseGame,
          resumePage: playGame,
      } ;

})();

       
