console.log( 'quiz.js' );

var quizPage = (function() {

var questionOrder ;
var quizContainer ;
var MAX_QUESTIONS = 20;
var numQuestions ;
var gradeButton ;
var ANSWER_CHARS = [ 'A', 'B', 'C', 'D', 'E' ];
var PRINT_VERSION = false ;

function pickAnswer( event ) {
	var button = event.target ;
    killEvent( event );
    if ( gradeButton.style.display == 'none' ) {
        return ;
    }
    var questionIndex = button.id.split( "_" )[1];
    var alreadySelected = (button.className == 'selectedAnswer' );
    for ( var i=0; i<ANSWER_CHARS.length; i++ ) {
       var obj = getAnswerObj( questionIndex, i );
       if ( obj ) {
           obj.className = 'unselectedAnswer' ;
       }
    }
    if ( !alreadySelected ) {
        button.className = "selectedAnswer" ; 
    }
}

function getSelectedAnswerIndex( questionIndex ) {
    for ( var i=0; i<ANSWER_CHARS.length; i++ ) {
       var obj = getAnswerObj( questionIndex, i );
       if ( obj && obj.className == 'selectedAnswer' ) {
            return i;
       }
    }
}





function addQuestion( questionIndex, questionField, answerField ) {
    var questionNumber = questionIndex + 1;
    var dataIndex = questionOrder[ questionIndex ];
    var row = studyStack.getRow( dataIndex );
    var qFieldId = studyStack.fields[ questionField ].id;
    var aFieldId = studyStack.fields[ answerField ].id ;
    var correctAnswer = row.getDataElement( aFieldId ).value;
    var questionText = row.getDataElement( qFieldId ).value ;

    var answers = getIncorrectAnswers( dataIndex, questionField, answerField );
    answers[ answers.length ] = {text: correctAnswer };
//    answers = answers.sort(function() {return 0.5 - Math.random()});    
    shuffle( answers );
    var answerHtml = '';
    var correctIndex = -1;
    for ( var i=0; i<answers.length; i++ ) {
       var name = 'name_' + questionIndex + '_' + i ;
       if ( answers[i].text == correctAnswer ) {
           correctIndex = i;
       }
//       answerHtml = answerHtml + '<button class="unselectedAnswer" id="' + name + '" onclick="quizPage.pickAnswer(this,event);return false;" >' + ANSWER_CHARS[i] + '.&nbsp; ' + escapeHtml(answers[i].text) + '</button><br />' ;
       answerHtml = answerHtml + '<button class="unselectedAnswer" id="' + name + '"  >' + ANSWER_CHARS[i] + '.&nbsp; ' + escapeHtml(answers[i].text) + '</button><br />' ;
    }

    var newDiv = document.createElement( 'div' );
    newDiv.innerHTML = '<div id="questForm' + questionIndex +'" class="quizProblem">' +
                          '<input type="hidden" value="' + correctIndex + '" id="correctIndex' + questionIndex + '" />' +
                          '<img id="x' + questionIndex + '" class="redX" src="images/redx.png" />' + 
//                          '<div class="questionNumber">' + questionNumber + '.</div>' +
                          '<div class="questionFieldValue">' + questionNumber + '.&nbsp;'  + escapeHtml(questionText) + '</div>' +
                          '<div class="quizAnswers">' + answerHtml + '</div>' +
                       '</div>';

    quizContainer.appendChild( newDiv );
}
function initQuiz( ) {
   quizContainer = document.getElementById( 'quizContainer' );
   if ( studyStack.getNumSides() < 2 || studyStack.getNumCards() < 4 ) {
        quizContainer.innerHTML = "<div id='activityError'>The Quiz activity requires at least four cards with at least two sides.</div>" ;
        return ;
   }
   questionOrder = createShuffledIntArray( studyStack.getNumCards() ) ;
   quizContainer.innerHTML = '';
   numQuestions = MAX_QUESTIONS ;
   if ( numQuestions > questionOrder.length ) {
       numQuestions = questionOrder.length ;
   }
   var numFirstHalf = Math.floor( numQuestions / 2 );
   addQuestions( 0, numFirstHalf, 0, 1 );
   addQuestions( numFirstHalf, numQuestions, 1, 0 );


   $( '.unselectedAnswer' ).click( pickAnswer );

   
   addExtraCredit( numQuestions, questionOrder.length );
   if ( ! PRINT_VERSION ) {
       gradeButton = document.createElement( 'button' );
       gradeButton.innerHTML = 'Grade Quiz' ;
       gradeButton.id = 'gradeQuizButton' ;
       gradeButton.onclick = gradeQuiz ;
       gradeButton.className = 'actionButton smallbutton';
       quizContainer.appendChild( gradeButton );
   }
}

function getAnswerObj( questionIndex, answerIndex ) {
      var name = 'name_' + questionIndex + '_' + answerIndex ;
      return document.getElementById( name );
}

function showCorrectAnswer( questionIndex ) {
        var answerIndex = document.getElementById( 'correctIndex' + questionIndex ).value;
        var correctObj = getAnswerObj( questionIndex, answerIndex );
        correctObj.className = 'correctAnswer';
}

function gradeQuiz() {
    var numCorrect = 0;
    for ( var i=0; i<numQuestions; i++ ) {
        var questForm = document.getElementById( 'questForm' + i );
        var correctIndex = document.getElementById( 'correctIndex' + i ).value;
        var selectedIndex = getSelectedAnswerIndex( i );
        var selectedObj = getAnswerObj( i, selectedIndex );
        if ( correctIndex == selectedIndex ) {
            numCorrect++;
            if ( selectedObj != null ) {
                selectedObj.className = 'correctAnswer';
            }
        }
        else {
            if ( selectedObj != null ) {
                selectedObj.className = 'incorrectAnswer';
            }
            var redX = document.getElementById( 'x' + i );
            redX.style.display = 'block';
            redX.title = (getAnswerObj( i, correctIndex ).innerHTML + "").replace( "&nbsp;", " " );
            redX.onclick = function() {
                    showCorrectAnswer( this.id.substr(1) ) ;
            };
        }
    } 
    var percent = Math.round( 100 * numCorrect / numQuestions ) ;
    var extraCreditInput = document.getElementById( 'extraCreditInput' );
    var ecCorrect = false ;
    if ( extraCreditInput ) {
         ecCorrect = document.getElementById( 'extraCreditInput' ).value.toUpperCase().trim() == extraCreditAnswer.toUpperCase().trim() ;
         if ( !ecCorrect ) {
              var ecX = document.getElementById( 'extraCreditX' );
              ecX.style.display = 'block';
              ecX.title = extraCreditAnswer ;
         }
    }
    var extraCredit = (ecCorrect) ? 5 : 0;
    var letterGrade = getGrade( percent + extraCredit);
    if ( percent + extraCredit > 80 ) {
          studyStack.finishActivity( QUIZ_CODE );
    }
    gradeButton.style.display = 'none';
    var gradeArea = document.createElement( "div" );
    gradeArea.className = 'gradeArea' ;
    var gradeNumber = percent + ((extraCredit == 0) ? '' : (' + ' + extraCredit));
    gradeArea.innerHTML = '<span class="letterGrade">' + letterGrade + '</span>&nbsp;&nbsp;<span class="percentGrade">' + gradeNumber + '%</span>';
    quizContainer.appendChild( gradeArea );
    if ( percent < 100 ) {
        var instructMsg = document.createElement( 'div' );
        instructMsg.id = 'instructMsg';
        instructMsg.innerHTML = 'Tap the red Xs to display correct answers.';
        gradeArea.appendChild( instructMsg );
    }
}


function getGrade( percent ) {
    if ( percent > 99 ) return "A+";
    if ( percent >= 90 ) return "A" ;
    if ( percent >= 80 ) return "B" ;
    if ( percent >= 70 ) return "C" ;
    if ( percent >= 60 ) return "D" ;
    return "F" ;
}

function addQuestions( start, stop, questionField, answerField ) {
   var questionFieldName = studyStack.fields[ questionField ].name;
   var answerFieldName = studyStack.fields[ answerField ].name;
   addInstructions( (PRINT_VERSION ? 'Circle the letter of' : 'Select'), questionFieldName, answerFieldName );
   for ( var i=start; i<stop; i++ ) {
           addQuestion( i, questionField, answerField );
   }
}

var extraCreditAnswer = '';

function addExtraCredit( start, end ) {
    // Using the remaining unused cards, look for one that would be appropriate for a 
    // fill-in-the-blank extra credit question
    for ( var i=start; i<end; i++ ) {
       var dataIndex = questionOrder[ i ];
       var row = studyStack.getRow( dataIndex );
       var startCol = Math.floor( Math.random() * studyStack.getNumSides() );
       for ( var c=0; c<studyStack.getNumSides(); c++ ) {
           var ecCol = (c + startCol) % studyStack.getNumSides();
           extraCreditAnswer = row.getDataElement( studyStack.fields[ ecCol ].id ).value;

           if ( goodExtraCreditAnswer( extraCreditAnswer ) ) {
               addExtraCreditQuestion( i, ecCol, start+1 ); 
               return;
           }
       }
    }
    extraCreditAnswer = '';
}

function addExtraCreditQuestion( orderIndex, answerColumn, qNum ) {
    var questionIndex = questionOrder[ orderIndex ];
    var questionNumber = qNum;
    var row = studyStack.getRow( questionIndex );
    var answer = row.getDataElement( studyStack.fields[ answerColumn ].id ).value;
    var maxLength=0;
    var questionColumn=-1;
    var MIN_QUESTION_LENGTH = 2;
    var question;

    for ( var i=0; i<studyStack.getNumSides(); i++ ) {
        question = row.getDataElement( studyStack.fields[ i ].id ).value;
        if ( (question != answer) && (question.length >= MIN_QUESTION_LENGTH) && (question.length > maxLength) ) {
             questionColumn = i;
             break;
        }
    }
    if ( questionColumn >= 0 ) {
        var questionFieldName = studyStack.fields[ questionColumn ].name;
        var answerFieldName = studyStack.fields[ answerColumn ].name;
        addInstructions( 'EXTRA CREDIT: ' + (PRINT_VERSION ? 'Give' : 'TYPE'), questionFieldName, answerFieldName );

        var newDiv = document.createElement( 'div' );
        var answerHtml = (PRINT_VERSION ? '' : ('<input type="text" value="" maxlength="' + MAX_EXTRA_CREDIT_ANSWER_LENGTH + '" id="extraCreditInput" />'));

        newDiv.innerHTML = '<form id="questForm' + questionIndex +'" class="quizProblem" onsubmit="quizPage.gradeQuiz(); return false;" >' +
                          //'<input type="hidden" value="' + answer + '" id="correctAnswer' + questionIndex + '" />' +
                          '<img id="extraCreditX" class="redX" src="images/redx.png" onclick="quizPage.showExtraCreditAnswer(event);" />' + 
                          '<div class="questionFieldValue">' + questionNumber + '.&nbsp;' + question + '</div>' +
                          '<div class="quizAnswers">' + answerHtml + '</div>' +
                          '<div id="extraCreditAnswer">' + extraCreditAnswer + '</div>' ;
                       '</form>';

        quizContainer.appendChild( newDiv );
    }
}


function showExtraCreditAnswer(event) {
     killEvent( event );
     document.getElementById( 'extraCreditAnswer' ).style.display = 'block';
}

var MAX_EXTRA_CREDIT_ANSWER_LENGTH=20;

function goodExtraCreditAnswer( answer ) {
     return answer.length>4 && answer.length<=MAX_EXTRA_CREDIT_ANSWER_LENGTH && answer.indexOf( ' ' )<0;
}

function addInstructions( verb, questionFieldName, answerFieldName ) {
    var newDiv = document.createElement( 'div' );
    newDiv.innerHTML = '<div class="quizInstructions">' +
                           verb + ' the ' + answerFieldName + ' that corresponds to the displayed ' + questionFieldName + '.' +
                       '</div>';

    quizContainer.appendChild( newDiv );
}

  
       
       function problemToString( problem ) {
            return '{ questionHeading:' + problem.questionHeading +
                     ', answerHeading:' + problem.answerHeading +
                     ', question: ' + problem.question +
                     ', correctAnswer: ' + problem.correctAnswer.text +
                     '}';
                     
       }
       
       function getIncorrectAnswers( dataIndex, questionField, answerField ) {
             var row = studyStack.getRow( dataIndex );
             var qFieldId = studyStack.fields[ questionField ].id;
             var aFieldId = studyStack.fields[ answerField ].id ;

             var answer = row.getDataElement( aFieldId ).value.trim();
             var question = row.getDataElement( qFieldId ).value.trim();

             var wrongAnswers = new Array();
             for ( var numTries =0; numTries < 50; numTries++ ) {
                  var p = Math.floor( Math.random() * studyStack.getNumCards() );
                  var wrongAnswer = studyStack.getCardSide( p, answerField ).trim();
                  var wrongQuestion = studyStack.getCardSide(p,questionField).trim();
                  if ( wrongAnswer != answer && wrongQuestion != question ) {
                      // make sure it's not one of the wrong answers we already have
                      var n ;
                      for ( n=0; n<wrongAnswers.length; n++ ) {
                           if ( wrongAnswer == wrongAnswers[n].text ) {
                               break;
                           }
                      }
                      if ( n == wrongAnswers.length ) {
                          // we made it to the end w/o breaking, so this answer hasn't been used yet
                          wrongAnswers[ wrongAnswers.length ] = {text: studyStack.getCardSide(p,answerField)};
                      }
                  }
                  if ( wrongAnswers.length == 3 ) {
                      break;
                  }
             }
             return wrongAnswers;             
       }


      function startPage() {
	   sharePath = '/quiz-' + studyStack.idOnStudyStack ;
	   contextMenu.setSetupFunc( setupContextMenu );
	         
	   initQuiz( true );
      }
      
  	function setupContextMenu() {
	    contextMenu.addMenuItem( "Restart", true, initQuiz );
	}
 
	
      return {
          gradeQuiz: gradeQuiz,
          pickAnswer: pickAnswer,
          showExtraCreditAnswer: showExtraCreditAnswer,
          startPage: startPage,
      } ;

})();

	
