console.log( 'search.js' );

var searchPage = (function() {
    var NUM_SEARCH_ITEMS_PER_REQUEST = 15;
    var searchButton = document.getElementById( 'searchButton' );
    var searchUser = document.getElementById( 'searchUser' );
    var searchText = document.getElementById( 'searchText' );
    var searchCategories = document.getElementById( 'searchCategories' );
    var searchResults = document.getElementById( 'searchResults' );
    var jqSearchPage = $( document.getElementById( 'searchPage' ) );
    var requestPage = 1;
    var loadingResultsDiv = null ;
    var searching = false ;
    
    searchButton.onclick = startSearch;
    searchUser.onchange = saveSearchUser ;
    searchText.onchange = saveSearchText ;
    searchCategories.onchange = saveSearchCategory ;
    
    searchCategories.innerHTML = '<option value="-1">any category</option>' + getCategoryOptionHtml()  ;
    $( searchCategories ).val( dataManager.getLocalItem( 'searchCategory', -1 ) );
        
    searchUser.value = dataManager.getLocalItem( 'searchUser', '' );
    searchText.value = dataManager.getLocalItem( 'searchText', '' );

	function startPage() {
       contextMenu.setSetupFunc( setupContextMenu );
	   setTitle( "StudyStack.com" );
	   if ( loadingResultsDiv == null ) {
	       createLoadingResultsDiv();
	   }
	}
	
	function createLoadingResultsDiv() {
	     loadingResultsDiv = document.createElement( 'div' );
	     loadingResultsDiv.id = 'loadingResultsDiv';
	     loadingResultsDiv.innerHTML = 'Loading results' +
				'<div class="spinner">' +
				  '<div class="rect1"></div>' +
				  '<div class="rect2"></div>' +
				  '<div class="rect3"></div>' +
				  '<div class="rect4"></div>' +
				  '<div class="rect5"></div>' +
				'</div>' ;	     
	}
	
	function saveSearchUser() {
	    dataManager.setLocalItem( 'searchUser', searchUser.value );
	}
	
	function saveSearchText() {
		dataManager.setLocalItem( 'searchText', searchText.value );
	}
	
	function saveSearchCategory() {
	     console.log( 'saveSearchCategory' + searchCategories.value );
	     dataManager.setLocalItem( 'searchCategory', searchCategories.value );
	}


    
    document.addEventListener("online", refreshConnectionStatusMessage, false);
    document.addEventListener("offline", refreshConnectionStatusMessage, false);

    
    function refreshConnectionStatusMessage() {
         var mysetsBottom = document.getElementById( 'mysetsBottom' ) ;
         if ( mysetsBottom ) {
	         if ( !hasNetworkConnection() ) {
	            mysetsBottom.innerHTML = 'Enable WIFI/Data connection to sync with studystack.com' ;
	         } 	         
	         else if ( isGuest() ) {
	            mysetsBottom.innerHTML = 'Login to sync with studystack.com' 
	                                     + '<div class="actionButton" onclick="mysetsPage.openConnectWindow()">Login</div>' ; 
	         }
	         else {
	            mysetsBottom.innerHTML = '' ;
	         }
         }
    }
    

	function setupContextMenu() {
	     // contextMenu.addMenuItem( "Logout", true, mysetsPage.logout );
		    contextMenu.addMenuItem( "Sort by set's name", true, sortByName );
		    contextMenu.addMenuItem( "Sort by date created", true, sortByDateCreated );

	}
	
	
	function startSearch() {
	    console.log( 'startSearch' );
	    searchResults.innerHTML = '' ;
	    requestPage = 1;
		continueSearch();
	}
	
	function startUserSearch( userName ) {
    	 searchUser.value = userName ; 
    	 dataManager.setLocalItem( 'searchUser', userName );
         searchText.value = '' ;
         dataManager.setLocalItem( 'searchText', '' );
    	 
	     clickMenuWithName( SEARCH_STUDYSTACK );
		 startSearch();
	}
	
	
	function continueSearch() {
	     console.log( 'continueSearch' );
	     searching = true ;
		 searchResults.appendChild( loadingResultsDiv );
	     webService.doSearch( searchUser.value, searchText.value, searchCategories.value, NUM_SEARCH_ITEMS_PER_REQUEST, requestPage, function( jsonResponse ) {
	           try {
	    	       searchResults.removeChild( loadingResultsDiv );
	           } catch ( err ) {}
	    	   if ( jsonResponse == null ) {
	    		   toast( "No response from server.  Check network settings." );
	    		   return ;
	    	   }
	     	   var results = jsonResponse.results ;
	     	   if ( results.length > 0 ) {
	     	       requestPage++;
	     	   }
	     	   for ( var i=0; i<results.length; i++ ) {
	     	        var result = results[i];
	     	        var stackId = result.docid;
	     	        var numCards = result.variable_0 ;
	     	        var numStars = result.variable_1 ;
	     	        var isExpert = result.variable_2 ;
	     	        var hasSlides = result.variable_3 ;
	     	        var category = result.category_category ;
	     	        var newDiv = document.createElement( 'div' );
	     	        var title = result.title ;
	     	        var description = result.description ;
	     	        if ( title == description ) {
	     	            description = '' ;
	     	        }
	     	        if ( title == '' || title.startsWith( 'Stack #' ) ) {
	     	            title = description ;
	     	            description = '' ;
	     	        }
	     	        newDiv.className = 'sR';
	     	        newDiv.onclick = function() {
	     	        	onClickSet( this );
	     	        }
	     	        newDiv.stackId = stackId ;
	     	        newDiv.innerHTML = "<div class='s1'>" + 
	     	                               "<div class='sT'>" + result.title + "</div>" +
	     	                               ((searchCategories.value >= 0) ? "" : ("<div class='sC'>" + category + "</div>")) +
	     	                               //((description=='') ? "" : ("<div class='sD'>" + result.description + "</div>"))+
	     	                           "</div>" +
	     	                           "<div class='s2'>" + 
	     	                               "<div class='sN'>" + result.snippet_text + "</div>" +
	     	                               "<div class='sU'>Created by: " + result.createdBy + "</div>" +
	     	                               "<div class='sNum'>" + numCards + " cards</div>" + 
	     	                           "</div>" ;
	     	                           
	     	        searchResults.appendChild( newDiv ) ;
	     	   }
	     	   searching = false ;
	     } );
	}
	
	function  onClickSet( obj ) {
		if ( syncService.syncOneSet( obj.stackId ) ) {
			clickMenuWithName( MY_SETS );
		}
	}
	
	function handleScroll( event ) {
	    if ( !searching && (jqSearchPage.scrollTop() + jqSearchPage.innerHeight() >= jqSearchPage[0].scrollHeight ) ) {
	        continueSearch();
	    }
	}
	
    jqSearchPage.bind( 'scroll', handleScroll );
	
	
      return {
          startPage: startPage,
          startSearch: startSearch,
          startUserSearch: startUserSearch,
      } ;

})();

	