console.log( 'signIn.js' );

var GUEST_ACCESS_TOKEN = "guest" ;
var accessToken = null ;
var userId = 0;
var FACEBOOK_APP_ID = "52462214664";

var genericServerError = "Unable to contact studystack.com servers" ;

function initializeSignIn() {

    accessToken = dataManager.getLocalItem( "accessToken" );
    userId = dataManager.getLocalItem( "userId", 0 );
    
    if ( accessToken == null ) {
        if ( hasNetworkConnection() ) {
        	   showConnectWindow();
        }
        else {
            setAccessToken( 0, GUEST_ACCESS_TOKEN ); 
        }   
    } 
    
    if ( hasNetworkConnection() ) {
        console.log( 'in initializeSignIn openUrl = ' + window.openUrl );
        if ( window.openUrl != null ) {
        	setTimeout( function() {
	       	    processOpenUrl( window.openUrl );
        	}, 0 );
        }	
	    else {
	        console.log( 'signIn.initializeSignIn calling syncService.start( true )' );
	        syncService.start( true );
        }
    }
}

function continueAsGuest() {
    setAccessToken( 0, GUEST_ACCESS_TOKEN );
    closeConnectWindow(); 
}



function isGuest() {
    return (accessToken == null) || (accessToken == GUEST_ACCESS_TOKEN) ;
}

var loginUserName  ;
var loginPassword ;
var loginButton ;
var resetPasswordButton ;
var facebookButton ;
var onLoginFunc = null ;

var signupUserName ;
var signupPassword ;
var signupEmail ;
var signupButton ;


function showConnectWindow( loginFunc ) {
    onLoginFunc = loginFunc ;
    document.getElementById( 'signInWrapper' ).style.display = 'table';
    loginUserName = document.getElementById( 'loginUserName' ) ;
    loginPassword = document.getElementById( 'loginPassword' );
    loginButton = document.getElementById( 'loginButton' );
    resetPasswordButton = document.getElementById( 'resetPasswordButton' );
    facebookButton = document.getElementById( 'fbInfo' );
    
    loginUserName.onkeyup = enableDisableLoginButtons ;
    loginPassword.onkeyup = enableDisableLoginButtons ;
    loginUserName.onchange = saveUserName ;
    resetPasswordButton.onclick = resetPassword ;
    facebookButton.onclick = showFbInfo ;
    loginButton.onclick = signIn ;
    
    signupUserName = document.getElementById( 'signupUserName' );
    signupPassword = document.getElementById( 'signupPassword' );
    signupEye = document.getElementById( 'signupEye' );
    signupEmail = document.getElementById( 'signupEmail' );
    signupButton = document.getElementById( 'signupButton' );
	signupButton.onclick = createAccount;
    
    signupUserName.onkeyup = enableDisableCreateAccountButton ;
    signupUserName.onchange = saveUserName ;
    signupPassword.onkeyup = enableDisableCreateAccountButton ;
    
    addTapListener( $(signupEye), showPassword );
    addTapReleaseListener( $(signupEye),  hidePassword );
    
    
    
    signupEmail.onkeyup = enableDisableCreateAccountButton ; 
}

function showPassword() {
	console.log('showPassword' );
	signupPassword.type = 'text' ;
}

function hidePassword() {
	console.log( 'hidePassword' );
	signupPassword.type = 'password';
}

function saveUserName( event ) {
     dataManager.setLocalItem( "userName", event.target.value );
}

function closeConnectWindow() {
    document.getElementById( 'signInWrapper' ).style.display = 'none';
}

function isConnectWindowDisplayed() {
    return $( '#signInWrapper' ).is(':visible') ;
}

function setAccessToken( id, token ) {
    accessToken = token ;
    dataManager.setLocalItem( "accessToken", token );
    
    userId = id ;
    dataManager.setLocalItem( "userId", id );
} 


function isSignupAreaDisplayed() {
    return $( '#signupArea' ).is(':visible') ;
}

function showSignupArea() {
	enableDisableCreateAccountButton();
    document.getElementById( 'signupArea' ).style.display = 'table'; 
}
function closeSignupArea() {
    document.getElementById( 'signupArea' ).style.display = 'none'; 
}   


function isLoginAreaDisplayed() {
    return $( '#loginArea' ).is(':visible') ;
}

function showLoginArea() {
    loginUserName.value = dataManager.getLocalItem( 'userName', '' );
    enableDisableLoginButtons();
    document.getElementById( 'loginArea' ).style.display = 'table'; 
}
function closeLoginArea() {
    document.getElementById( 'loginArea' ).style.display = 'none'; 
}   


function enableDisableLoginButtons() {
	 var disable = ( loginUserName.value.length == 0 ) ;
     resetPasswordButton.disabled = disable ;
     addRemoveClass( resetPasswordButton, disable, 'disabledButton' );
     
     disable = ( loginUserName.value.length == 0 || loginPassword.value.length == 0 ) ;
     loginButton.disabled = disable ;
     addRemoveClass( loginButton, disable, 'disabledButton' );
}

function enableDisableCreateAccountButton() {
    var disable = signupUserName.value.trim().length == 0 ||
                            signupPassword.value.length == 0 ||
                            signupEmail.value.trim().length == 0 ;
    signupButton.disabled = disable ;
    addRemoveClass( signupButton, disable, 'disabledButton' );
}

function resetPassword( event ) {
   killEvent( event );
   webService.resetPassword( loginUserName.value, function( error ) {
   		if ( error == NO_ERROR ) {
            toast( "An email with a temporary password has been sent to the address associated with your StudyStack account.", 10000 );
   		}
   		else if ( error == NO_USERS ) {
            toast( "No users found" );
   		}
   		else if ( error == INVALID_EMAIL ) {
            toast( "The email address associated with your StudyStack account is invalid." );
   		}
   		else if ( error == MISSING_EMAIL ) {
            toast( "There is no email address associated with your StudyStack account." );
   		}
   		else {
           toast( genericServerError );
   		}
   } );
}

function showFbInfo() {
    toast( "This app does not currently support logging in via Facebook.   However, if you login to your account on studystack.com via facebook, you can setup a username and password that you can use with this app.   Just login on studystack.com (using the \"Login with Facebook\" button) and click the Account tab.   Then enter a username and password.   After that you'll be able to login using either your username and password or the facebook connect button.", 600000 ); 
}
   

function handleSignIn( error, userId, token ) {
      console.log( 'handleSignIn error=' + error );
      if ( error == NO_ERROR && token.length > 0 && userId > 0 ) {
         setAccessToken( userId, token );
         closeLoginArea();
         closeSignupArea();
         closeConnectWindow();
         if ( onLoginFunc ) {
             onLoginFunc() ;
         }
         console.log( 'signIn.handleSignIn calling syncService.start( true )' );
         syncService.start( true );
         return ;
      }
      else if ( error == BAD_LOGIN ) {
          toast( "Login failed", 2000 );
      }
      else if ( error == USERNAME_TAKEN ) {
          toast( "Username is already taken" );
      }
      else if ( error == INVALID_EMAIL ) {
          toast( "Invalid email address" );
      }
      else {
           toast( genericServerError );
           console.log( 'error=' + error + ' userId=' + userId + ' token=' + token );
      }
} 

function signIn(event) {
    killEvent( event );
    console.log( 'signIn' );
	webService.signIn( loginUserName.value, loginPassword.value, handleSignIn );
}


function submitLogin() {
     console.log( 'submitLogin' );
     if ( loginUserName.value.length == 0 ) {
          loginUserName.focus();
          return false ;
     }
     else if ( loginPassword.value.length == 0 ) {
          loginPassword.focus();
          return false ;
     }
     else {
          signIn();
          return true ;
     }
}


function createAccount() {
    if ( signupUserName.value.indexOf( '@' )>=0 ) {
         toast( "Invalid @ character in username." );
         signupUserName.focus();
    }
    else if ( signupEmail.value.indexOf( '@' ) < 0 || signupEmail.value.indexOf( '.' ) < 0 ) {
         toast( "Invalid email address" );
         signupEmail.focus() ;
    }
    else {
		webService.createAccount( signupUserName.value.trim(), signupPassword.value, signupEmail.value.trim(), handleSignIn );
    }
    
}