console.log( 'sync.js' );

var syncService = (function() {
    var DELETE = 0;
    var ADD = 1;
    var MODIFY = 2;
    var MARK_CLEAN = 3;
    
    var STACK = 1;
    var FIELD = 2;
    var DATA = 3;

    var busyIndicator = document.getElementById( 'busyIndicator' );
    var setsModifiedOnServerList = null ;
    var running = false ;
    
    var localStacksChanged = [];
    var serverStacksChanged = [];
    
    function start( pullUpdatesFromStudyStack ) {
         if ( hasNetworkConnection() && !isGuest() ) { 
             if ( !running ) {
                startBusyAnimation();
	         	localStacksChanged = [];
	         	serverStacksChanged = [];
	         	sendLocalUpdatesToStudyStack( pullUpdatesFromStudyStack );
	         }
	     }
    }
    
    function startBusyAnimation() {
        running = true ;
     	busyIndicator.style.display = 'block' ;
    }
    
    function addToSet( setVar, value ) {
         if ( -1 == setVar.indexOf( value ) ) {
              setVar[ setVar.length ] = value ;
         }
    }
    
    function sendLocalUpdatesToStudyStack( pullUpdatesFromStudyStack ) {
          if ( isGuest() ) {
              onDone();
              return ;
          }
          
          // get one stack from our local DB that is dirty
          console.log( 'calling getDirtyStack' );
          dataManager.getDirtyStack( function( stack ) {
                 console.log( 'getDirtyStack calledBack with stack=' + stack );
                 if ( stack != null ) {
                    // send updates to studystack
                    console.log( 'calling updateDirtyParts' );
                    webService.updateDirtyParts( stack, function( updatedIds ) {
                         console.log( 'updateDirtyParts called callback with ' + updatedIds );
                         if ( updatedIds ) {
                             addToSet( serverStacksChanged, stack.id );
                             stack.markClean(); 
	                         // loop through updatedIds and update local DB to clear dirty flags and set idsOnStudyStack
	                         console.log( 'calling saveIds' );
							 saveIds( updatedIds, function() {
							     console.log( 'savedIds call back called.  calling sendLocalUpdatesToStudyStack recursively' );
		                         // recursively call this function to process next dirty stack
		                         sendLocalUpdatesToStudyStack();
							 } );
                         }
                         else {
                         	//toast( 'Failed to send stack ' + stack.name + ' to StudyStack server.', 4000 );
                            finishSendLocal( pullUpdatesFromStudyStack );
                         }
                    } );
                 }
                 else {
                     finishSendLocal( pullUpdatesFromStudyStack );
                 }
          } ); 
    }
    
    function finishSendLocal( pullUpdatesFromStudyStack ) {
		if ( pullUpdatesFromStudyStack ) {
              console.log( 'calling getUpdatesFromStudyStack-------------------------------------------------------------------' );
              getUpdatesFromStudyStack();
         }
         else {
            console.log( 'sendLocalUpdatesToStudyStack calling onDone' ); 
         	onDone();
         }    	
    }
    
    function saveIds( updatedIds, completeCallback ) {
        if ( updatedIds == null || updatedIds.length == 0 ) {
            console.log( 'saveIds calling completeCallback' );
        	setTimeout( completeCallback, 0 );
        }
        else {
            console.log( 'in saveIds   length=' + updatedIds.length );
        	// remove first pair
        	var idPair = updatedIds.splice( 0, 1 )[0];
            var key = idPair[0];
            var idOnStudyStack = idPair[1];
            var keyType = key.substring( 0, 1 );
            var localId = key.substring( 1 );
            
            dataManager.updateIdOnStudyStack( keyType, localId, idOnStudyStack, function() {
                console.log( 'updatedIdOnStudyStack called saveIds' );
            	saveIds( updatedIds, completeCallback );
            } );
    	}
    }
    
    function getUpdatesFromStudyStack() {
          if ( isGuest() ) {
              onDone();
              return ;
          }
    
          // get list of all sets from our DB along with the lastModOnStudyStack value
          dataManager.getLastModifiedStackList( function( idList, modTimeList ) {
              console.error( 'dataManager.getLastModifiedStackList returned idList=' + idList + ' modTimeList=' + modTimeList );
              // send this list to StudyStack
               webService.getUpdatedSets( idList, modTimeList, 
                  function( ids ){
                      // StudyStack will return a list of setIds that have changed and any new 
                      // ones that have been created/favorited by the user with the specified accessToken
                      setsModifiedOnServerList = ids;
                      setTimeout( syncNextSet, 0 );
                  },
                  handleServerError );
          } ); 
    }
    
    function handleServerError( jqXHR, textStatus, errorThrown ) {
       if ( errorThrown == "Unauthorized" ) {
            toast( 'Login failed.  Please relogin from the My Sets page', 4000 );
            setAccessToken( 0, GUEST_ACCESS_TOKEN );
       }
       else {
       	    console.log( 'handleServerError ' + getProps( errorThrown ) + ' status=' + textStatus );
       }
       onDone();
    }
    
    
    
    function syncOneSet( serverSetId ) {
    	if ( running ) {
    		toast( 'Busy updating.   Please try again.', 4000 );
    		return false ;
    	}
    	else {
    		startBusyAnimation();
        	setsModifiedOnServerList = [];
        	setsModifiedOnServerList[0] = serverSetId ;
        	syncNextSet();
        	return true ;
        }
    }
    
    
    
    function syncNextSet() {
        if ( !hasNetworkConnection() ) {
              onDone();
              return ;
        }

        if ( setsModifiedOnServerList.length > 0 ) {
        	var setToSync = setsModifiedOnServerList.splice( 0, 1 )[0];
        	if ( setToSync == "" || setToSync < 0 ) {
               	setTimeout( syncNextSet, 0 );
        	}
        	else { 
              // retrieve the setToSync from studystack.com
              webService.getStudyStack( setToSync, function( stack ) {
                  console.log( 'got stack ' + stack + ' from studystack.com' );
                  if ( stack == null ) {
                        toast( "Unable to get data from server for stack " + setToSync );
                        console.error( "Unable to get data from server for stack " + setToSync );
    	              	setTimeout( syncNextSet, 0 );
                  }
                  else {
                  	stack.id = setToSync ;
                  	// on complete
                  	// merge set into local database
			      	mergeStackIntoDatabase( stack, function() {
	                  	// then trigger next set to get synced 
    	              	setTimeout( syncNextSet, 0 );
			      	} );
			      }                  
              } );
           }
        }
        else {
             onDone();
        }
    }
    
    function onDone() {
         // stop the syncing animation
         busyIndicator.style.display = 'none';
         running = false ;
         console.log( 'sync done!!!!!!' );
         
         var msg = '';
         var toastDelay = 0;
         if ( localStacksChanged != null && localStacksChanged.length > 0 ) {
               msg += 'Changes received from studystack.com.' ;
               toastDelay += 3000;
         }  
         if ( msg != '' ) {
         	toast( msg, toastDelay );
     	 }
    }
    
    function mergeStackIntoDatabase( serverStack, complete ) {
    	dataManager.getStackWithServerId( serverStack.id, function( localStack ) {
    	    console.log( 'got localStack with serverId ' + serverStack.id + ' localStack=' + localStack );
    		if ( localStack == null ) {
    			importServerStack( serverStack, complete );
    		}
    		else {
    			mergeLocalAndServerStacks( localStack, serverStack, complete ); 
    		}
    	} );	
    }
    
    
    function importServerStack( serverStack, complete ) {
        console.log( 'importServerStack' );
    	dataManager.insertStudyStack( serverStack.name, serverStack.id, serverStack.category, serverStack.lastModified, 
    	                              serverStack.createdBy, serverStack.created, 0, 1, serverStack.options, serverStack.color, 
    	    function( newId ) {
    	        if ( newId > 0 ) {
				   addToSet( localStacksChanged, newId );    	        
                   var newStack = new StudyStack ( newId, serverStack.name, '', serverStack.id, serverStack.category, serverStack.lastModified, 0, null, 
                   									serverStack.createdBy, serverStack.created, 0, 1, '0' );
                   importFields( serverStack, newStack, function(){
                        importCards( serverStack, newStack, function() {
                             setTimeout( function() { complete( newStack ); }, 0 );
                        } );
                   } );
    	        }
    	        else {
             		setTimeout( function() { complete( null ); }, 0 );
                }
    	    } );
    }
    
    function importFields( serverStack, newStack, complete ) {
         importNextField( serverStack, newStack, complete );
    }
    
    function importNextField( serverStack, localStack, complete ) {
         var side = localStack.getNumSides();
         if ( side >= serverStack.columnNames.length ) {
               setTimeout( complete, 0 );
         }
         else {
          	  var nextField = new StudyField( -1, serverStack.columnNames[ side ], serverStack.fieldIds[ side ], 0 ) ;
          	  dataManager.insertStudyField( localStack.id, nextField, function() {
          	      localStack.addField( nextField );
          	      importNextField( serverStack, localStack, complete );
          	  } );
         }
    }
    
    function importCards( serverStack, localStack, complete ) {
          importNextCard( serverStack, localStack, 0, complete );
    }
    
    function importNextCard( serverStack, localStack, cardIndex, complete ) {
         if ( cardIndex >= serverStack.numCards ) {
              setTimeout( complete, 0 );
              localStack.markClean();
         }
         else {
              importCardsData( serverStack, localStack, cardIndex, 
              	 function() {
              	     importNextCard( serverStack, localStack, cardIndex+1, complete );
              	 } );
         }
    }
    
    function importCardsData( serverStack, localStack, cardIndex, complete ) {
         importNextData( serverStack, localStack, cardIndex, 0, complete );
    }
    
    function importNextData( serverStack, localStack, cardIndex, columnIndex, complete ) {
         if ( columnIndex >= localStack.getNumSides() ) {
         	setTimeout( complete, 0 );
         }
         else {
               var row = localStack.getRowByCardId( cardIndex );
               var dataElement = row.getDataElement( localStack.fields[ columnIndex ].id );
               dataElement.value = serverStack.data[ cardIndex ][ columnIndex ];
               dataElement.cardIdOnStudyStack = serverStack.cardIds[ cardIndex ];
               dataElement.dirty = 0; 
         	   dataManager.insertDataElement( localStack.id, row.cardId, localStack.fields[ columnIndex ].id, dataElement, function() {
            	   importNextData( serverStack, localStack, cardIndex, columnIndex + 1, complete );
         	   } ) ;
         }
    }
    
    function mergeLocalAndServerStacks( localStack, serverStack, complete ) {
        console.log( 'mergeLocalAndServerStacks' );
        var originallyClean = !localStack.dirty ;
    	// first build a list of the changes that need to be made
    	var updatesNeeded = [];
    	addStackDifferences( localStack, serverStack, updatesNeeded );
    	addFieldDifferences( localStack, serverStack, updatesNeeded );
    	addDataDifferences( localStack, serverStack, updatesNeeded );
    	
    	
    	if ( originallyClean ) {
    	     // localStack was originally clean, but doing the above changes will 
    	     // mark it dirty.   However since these changes came from the server, let's
    	     // reset it back to clean.
    	     addUpdateCommand( updatesNeeded, MARK_CLEAN, STACK, localStack.id );
    	}

		if ( updatesNeeded.length > 0 ) {
			addToSet( localStacksChanged, localStack.id );		
		}
    	
    	// now make the changes and call the "complete" method when done 
    	mergeNextChange( updatesNeeded, complete );
    }	


     function addUpdateCommand( list, command, targetType, targetElement ) {
     	list[ list.length ] = { command: command, targetType: targetType, targetElement: targetElement } ;
     } 

   	 function addStackDifferences( localStack, serverStack, updatesNeeded ) {
   	 	addUpdateCommand( updatesNeeded, MODIFY, STACK, { name: serverStack.name, description: serverStack.description, 
   	 		idOnStudyStack: serverStack.id,  category: serverStack.category, lastModOnStudyStack: serverStack.lastModified, 
   	 		dirty: (localStack.dirty?1:0), numCards: localStack.numCards, nextCardId: localStack.nextCardId, 
   	 		options: serverStack.options, id: localStack.id } );
   	 } 

   	 function addFieldDifferences( localStack, serverStack, updatesNeeded ) { 
    	var f1, f2;
    	
    	// check to see if any columns need to be added or deleted
    	var serverFieldIds = serverStack.fieldIds;
    	var serverFieldNames = serverStack.columnNames ;
    	for ( f1=0; f1<serverFieldIds.length; f1++ ) {
    		var serverId = serverFieldIds[f1] ;
    		var localField = localStack.getFieldByServerId( serverId );
   			var serverFieldName = serverFieldNames[f1] ;
    		if ( localField != null ) {
    			if ( localField.name !=  serverFieldName ) {
   					addUpdateCommand( updatesNeeded, MODIFY, FIELD, { name: serverFieldName, idOnStudyStack: serverId, id: localField.id, dirty: 0 } );
    			}
    		}
    		else {
    			// no match found 
				addUpdateCommand( updatesNeeded, ADD, FIELD, { studyStackId: localStack.id, name: serverFieldName, idOnStudyStack: serverId, dirty: 0 } );
			}
		}
		
    	// loop through each field of the localStack
    	for ( f1=0; f1<localStack.fields.length; f1++ ) {
    		var localField = localStack.fields[f1];
    		if ( localField.idOnStudyStack > 0 ) {
    		    var foundMatch = false ;
    			for ( f2=0; f2<serverFieldIds.length; f2++ ) {
    				if ( serverFieldIds[f2] == localField.idOnStudyStack ) {
    					foundMatch = true ;
    					break;
    				}
    			}
    			if ( !foundMatch ) {
					addUpdateCommand( updatesNeeded, DELETE, FIELD, { studyStackId: localStack.id, id: localField.id } ); 
   				}
    		}
    	}

	}


    function addDataDifferences( localStack, serverStack, updatesNeeded ) {
        var row; 
		for ( row=0; row<serverStack.cardIds.length; row++ ) {
			var serverCardId = serverStack.cardIds[ row ];
			var serverRowValues = serverStack.data[ row ];
			    	
    		// find the row in the localStack with the matching idOnStudyStack
    		var localRow = localStack.findRowByServerCardId( serverCardId );
    		console.log( 'addDataDifferences serverCardId=' + serverCardId + ' localRow=' + localRow );
			for ( var serverCol=0; serverCol<serverStack.fieldIds.length; serverCol++ ) {
				var serverFieldId = serverStack.fieldIds[ serverCol ]; 
				var serverDataValue = serverRowValues[ serverCol ];
				var localField = localStack.getFieldByServerId( serverFieldId );
				var localDataElement = null ;
				if ( localRow != null && localField != null ) {
					localDataElement = localRow.getDataElement( localField.id );
				}
				if ( localDataElement != null && (localDataElement.id >= 0)) {
					if ( serverDataValue != localDataElement.value ) {
						localDataElement.dirty = 0;
						localDataElement.value = serverDataValue ;
						addUpdateCommand( updatesNeeded, MODIFY, DATA, localDataElement );
					}
				}
				else {
				    if ( localRow == null ) {
				    	localRow = localStack.getRowByCardId( -1 );
				    	console.log( 'created new row ' + getProps( localRow ) );
				    	/*
				    	if ( localField != null ) {
				    		localRow.addDataElement( localField.id, new DataElement( -1, serverDataValue, -1, serverCardId, 0 ) );
				    	}
				    	*/
				    }
				    if ( localRow != null ) {
				    	console.log( 'addUpdateCommand for data difference value=' + serverDataValue );
						addUpdateCommand( updatesNeeded, ADD, DATA,	{
								value: serverDataValue,	
								stackId: localStack.id,	
								position: 0,
								cardId: localRow.cardId, 
								serverFieldId: serverFieldId,
								cardIdOnStudyStack: serverCardId, 
								dirty: 0 ,
								fieldId: localField.id
							});
					}
				}
			}
		}
		
    	// for each row in the localStack
    	for ( row=0; row<localStack.rows.length; row++ ) {
    		var localRow = localStack.rows[ row ];
    		var serverCardId = localRow.getCardIdOnStudyStack();
    		if ( serverCardId>=0 ) {
    			// see if that serverCardId still exists on the serverStack
    			var foundMatch = false ;
    			for ( var serverRow = 0; serverRow<serverStack.cardIds.length; serverRow++ ) {
    				if ( serverStack.cardIds[ serverRow ] == serverCardId ) {
    					foundMatch = true ;
    					break;
    				}
    			}
    			if ( !foundMatch ) {
					addUpdateCommand( updatesNeeded, DELETE, DATA, { stackId: localStack.id, cardId: localRow.cardId } );
				}
    		}
		}
	}

    
    function mergeNextChange( mergeChanges, complete ) {
        if ( mergeChanges.length == 0 ) {
             setTimeout( complete, 0 );
        }
        else {
    		var nextChange = mergeChanges.splice( 0, 1 )[0];
    		mergeChange( nextChange, 
    			function() { 
    				mergeNextChange( mergeChanges, complete ); 
    			} );          
        }
    }
    	
    
    function mergeChange( changeNeeded, complete ) {
    	// do the change
    	console.log( 'mergeChange command= ' + changeNeeded.command + ' targetType=' + changeNeeded.targetType );
    	if ( changeNeeded.targetType == DATA ) {
    		var targetElement = changeNeeded.targetElement ;
    		if ( changeNeeded.command == ADD ) {
				dataManager.insertDataElement( targetElement.stackId, targetElement.cardId, targetElement.fieldId, targetElement, complete );
    		}
    		else if ( changeNeeded.command == MODIFY ) {
    			dataManager.updateDataElement( targetElement, complete) ;
    		}
    		else if ( changeNeeded.command == DELETE ) {
    			dataManager.markForDeletion( targetElement.stackId, targetElement.cardId, complete );
    		}
    		else {
    			alert( 'unknown command ' + changeNeeded.command );
		    	setTimeout( complete, 0 );
    		}
    	}
    	else if ( changeNeeded.targetType == FIELD ) {
    		var targetElement = changeNeeded.targetElement ;
    		if ( changeNeeded.command == ADD ) {
				dataManager.insertStudyField( targetElement.studyStackId, targetElement, complete );
    		}
    		else if ( changeNeeded.command == MODIFY ) {
    			dataManager.updateStudyField( targetElement, complete) ;
    		}
    		else if ( changeNeeded.command == DELETE ) {
    			dataManager.markFieldForDeletion( targetElement.studyStackId, targetElement.id, complete );
    		}
    		else {
    			alert( 'unknown command ' + changeNeeded.command );
		    	setTimeout( complete, 0 );
    		}
    	}
    	else if ( changeNeeded.targetType == STACK ) {
    	    if ( changeNeeded.command == MARK_CLEAN ) {
    	    	dataManager.updateField( 'studystack', 'dirty', 0, changeNeeded.targetElement, complete );
    	    }
    	    else {
    			dataManager.updateStudyStack( changeNeeded.targetElement, complete );
    		}
    	}
    	else {
    		alert( 'unknown targetType ' + changeNeeded.targetType );
	    	setTimeout( complete, 0 );
    	} 
    }


    return {
              start: start,
              syncOneSet: syncOneSet,
    } ;


})();

	